# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-10 14:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0014_auto_20161209_2045'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order_item',
            name='size',
            field=models.CharField(blank=True, choices=[('', ''), ('R', 'Regular'), ('L', 'Large')], max_length=1, null=True, verbose_name='size'),
        ),
    ]



from django.contrib import admin
from souschef.order.models import Order, Order_item, OrderStatusChange


def make_delivered(modeladmin, request, queryset):
    queryset.update(status='D')
make_delivered.short_description = "Mark selected orders as delivered"


class OrderItemInline(admin.TabularInline):
    model = Order_item


class OrderItemAdmin(admin.ModelAdmin):
    search_fields = ['order__id']
    list_display = (
        'order',
        'order_item_type',
        'component_group',
        'total_quantity',
        'billable_flag',
        'price',
    )


class OrderAdmin(admin.ModelAdmin):
    search_fields = ['client__member__lastname', 'client__member__firstname']
    list_filter = ('status', 'delivery_date')
    list_display = (
        'id',
        'client',
        'status',
        'price',
        'delivery_date',
        'creation_date',
    )
    actions = [make_delivered]
    inlines = [
        OrderItemInline
    ]

admin.site.register(Order, OrderAdmin)
admin.site.register(Order_item, OrderItemAdmin)
admin.site.register(OrderStatusChange)



from django.core.management.base import BaseCommand
from souschef.member.factories import ClientFactory
from django.core.management import call_command


class Command(BaseCommand):
    help = 'Creates a happy bunch of clients without orders'

    def handle(self, *args, **options):
        # Load fixtures for the routes
        fixture_filename = 'routes.json'
        call_command('loaddata', fixture_filename)

        client = ClientFactory.create_batch(10)
        self.stdout.write(
            self.style.SUCCESS(
                'Successfully created client "%s"' % client
            )
        )



from django.db import models
from django.utils.translation import ugettext_lazy as _

COMPONENT_GROUP_CHOICES = (
    ('main_dish', _('Main Dish')),
    ('dessert', _('Dessert')),
    ('diabetic', _('Diabetic')),
    ('fruit_salad', _('Fruit Salad')),
    ('green_salad', _('Green Salad')),
    ('pudding', _('Pudding')),
    ('compote', _('Compote')),
    ('sides', _('Sides')),
)

COMPONENT_GROUP_CHOICES_MAIN_DISH = COMPONENT_GROUP_CHOICES[0][0]
COMPONENT_GROUP_CHOICES_SIDES = COMPONENT_GROUP_CHOICES[7][0]

INGREDIENT_GROUP_CHOICES = (
    ('meat', _('Meat')),
    ('dairy', _('Dairy')),
    ('fish', _('Fish')),
    ('seafood', _('Seafood')),
    ('veggies_and_fruits', _('Veggies and fruits')),
    ('legumineuse', _('Legumineuse')),
    ('grains', _('Grains')),
    ('fresh_herbs', _('Fresh herbs')),
    ('spices', _('Spices')),
    ('dry_and_canned_goods', _('Dry and canned goods')),
    ('oils_and_sauces', _('Oils and sauces')),
)

RESTRICTED_ITEM_GROUP_CHOICES = (
    ('meat', _('Meat')),
    ('vegetables', _('Vegetables')),
    ('seafood', _('Seafood')),
    ('seeds and nuts', _('Seeds and nuts')),
    ('other', _('Other')),
)


class Ingredient(models.Model):
    class Meta:
        verbose_name_plural = _('ingredients')

    # Information about the ingredient (in the recipe of a component)
    name = models.CharField(
        max_length=50,
        verbose_name=_('name'),
        unique=True,
    )

    description = models.TextField(
        verbose_name=_('description'),
        blank=True,
        null=True,
    )

    ingredient_group = models.CharField(
        max_length=100,
        choices=INGREDIENT_GROUP_CHOICES,
        verbose_name=_('ingredient group')
    )

    def __str__(self):
        return self.name


class Component(models.Model):

    class Meta:
        verbose_name_plural = _('components (Dishes and recipes)')

    # Meal component (ex. main dish, vegetable, seasonal) information
    name = models.CharField(
        max_length=50,
        verbose_name=_('name'),
        unique=True,
    )

    description = models.TextField(
        verbose_name=_('description'),
        blank=True,
        null=True,
    )

    component_group = models.CharField(
        max_length=100,
        choices=COMPONENT_GROUP_CHOICES,
        verbose_name=_('component group')
    )

    def __str__(self):
        return self.name

    @staticmethod
    def get_day_ingredients(component_id, delivery_date):
        """Returns a list of the actual ingredients
        of a component for the delivery date."""
        q = Component_ingredient.objects.\
            select_related('ingredient').\
            filter(component__id=component_id, date=delivery_date)
        return [ci.ingredient for ci in q]

    @staticmethod
    def get_recipe_ingredients(component_id):
        """Returns a list of the ingredients in the recipe of a component."""
        q = Component_ingredient.objects.\
            select_related('ingredient').\
            filter(component__id=component_id, date=None)
        return [ci.ingredient for ci in q]


class Component_ingredient(models.Model):
    component = models.ForeignKey(
        'meal.Component',
        verbose_name=_('component'),
        related_name='+',
        on_delete=models.CASCADE
    )

    ingredient = models.ForeignKey(
        'meal.Ingredient',
        verbose_name=_('ingredient'),
        related_name='+',
        on_delete=models.CASCADE
    )

    date = models.DateField(
        verbose_name=_('date'),
        blank=True,
        null=True,
    )

    def __str__(self):
        if self.date:
            return "<{}> includes <{}> on <{}>". \
                format(self.component.name,
                       self.ingredient.name,
                       self.date)
        else:
            return "<{}> contains <{}>". \
                format(self.component.name,
                       self.ingredient.name)


class Restricted_item(models.Model):

    class Meta:
        verbose_name_plural = _('restricted items (Restriction categories)')

    # Information about restricted item categories that some clients never eat
    #   for allergy or other reasons (ex. Gluten, Nuts, Pork)
    name = models.CharField(
        max_length=50,
        verbose_name=_('name')
    )

    description = models.TextField(
        verbose_name=_('description'),
        blank=True,
        null=True,
    )

    restricted_item_group = models.CharField(
        max_length=100,
        choices=RESTRICTED_ITEM_GROUP_CHOICES,
        verbose_name=_('restricted item group')
    )

    ingredients = models.ManyToManyField(
        'meal.Ingredient',
        through='Incompatibility'
    )

    def __str__(self):
        return self.name


class Incompatibility(models.Model):

    restricted_item = models.ForeignKey(
        'meal.Restricted_item',
        verbose_name=_('restricted item'),
        related_name='+',
        on_delete=models.CASCADE
    )

    ingredient = models.ForeignKey(
        'meal.Ingredient',
        verbose_name=_('ingredient'),
        related_name='+',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return "{} <clash> {}".format(self.restricted_item.name,
                                      self.ingredient.name)


class Menu(models.Model):

    class Meta:
        verbose_name_plural = _('menus')

    # Menu information for a specific date
    date = models.DateField(verbose_name=_('date'))

    components = models.ManyToManyField(
        'meal.Component',
        through="Menu_component"
    )

    def __str__(self):
        return "Menu for {}".format(str(self.date))

    @staticmethod
    def create_menu_and_components(menu_date, dish_names):
        """Create the menu and its components for the date.

        Parameters:
          menu_date : date on which menu will apply
          dish_names : a list of names of dishes included in the menu

        Behavior:
          - if Menu(s) already exists for the date, only one is kept and
            all its menu components are replaced by the new ones

        Raises :
          - Exception if dish does not exist
          - Exception if more that one dish per component_group
          - Exception if no dish for COMPONENT_GROUP_CHOICES_MAIN_DISH

        Returns:
          Number of menu components created.
        """

        num_menu_comp_created = 0
        component_groups = {}
        menus = Menu.objects.filter(date=menu_date)
        if not menus:
            menu = Menu(date=menu_date)
            menu.save()
        else:
            # menu(s) existed, must flush their menu_components
            Menu_component.objects.filter(menu__in=menus).delete()
            # flush extra menus
            for menu in menus[1:]:
                Menu.objects.filter(id=menu.id).delete()
            # reuse the first one
            menu = menus[0]
        for dish_name in dish_names:
            components = Component.objects.filter(name=dish_name)
            if not components:
                raise Exception(
                    "Component ", dish_name, " does not exist")
            component = components[0]
            if component_groups.get(component.component_group):
                raise Exception(
                    "Menu can only have one dish for component group " +
                    component.component_group)
            component_groups[component.component_group] = dish_name
            menu_component = Menu_component(menu=menu, component=component)
            menu_component.save()
            num_menu_comp_created += 1
        # END FOR
        if not component_groups.get(COMPONENT_GROUP_CHOICES_MAIN_DISH):
            raise Exception(
                "Menu must include one dish for component group " +
                COMPONENT_GROUP_CHOICES_MAIN_DISH)
        return num_menu_comp_created


class Menu_component(models.Model):
    menu = models.ForeignKey(
        'meal.Menu',
        verbose_name=_('menu'),
        related_name='+',
        on_delete=models.CASCADE
    )

    component = models.ForeignKey(
        'meal.Component',
        verbose_name=_('component'),
        related_name='+',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return "On {} <menu includes> {}".format(str(self.menu.date),
                                                 self.component.name)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-26 17:55
from __future__ import unicode_literals

from datetime import date

import annoying.fields
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations, models
import django.db.models.deletion


def migrate_relationships(apps, schema_editor):
    """
    Referencing --> Relationship
    EmergencyContact --> Relationship
    """
    Relationship = apps.get_model('member', 'Relationship')
    Referencing = apps.get_model('member', 'Referencing')
    EmergencyContact = apps.get_model('member', 'EmergencyContact')

    from souschef.member.models import Relationship as _ref_Relationship  # noqa

    for ref in Referencing.objects.all().order_by('pk'):
        print("Converting member.Referencing #{} to member.Relationship...".format(ref.pk))
        # Attention: the Rel object may exist because a member can be both Ref and Emgc
        try:
            rel = Relationship.objects.get(member=ref.referent, client=ref.client)
        except ObjectDoesNotExist:
            rel = Relationship(member=ref.referent, client=ref.client, type=[])

        if _ref_Relationship.REFERENT not in rel.type:
            rel.type.append(_ref_Relationship.REFERENT)
        rel.nature = rel.nature or "??? (migration 0032)"
        rel.extra_fields = rel.extra_fields or {}
        rel.extra_fields['referral_date'] = ref.date
        rel.extra_fields['referral_reason'] = ref.referral_reason
        rel.save()

    for emgc in EmergencyContact.objects.all().order_by('pk'):
        print("Converting member.EmergencyContact #{} to member.Relationship...".format(emgc.pk))
        # Attention: the Rel object may exist because a member can be both Ref and Emgc
        try:
            rel = Relationship.objects.get(member=emgc.member, client=emgc.client)
        except ObjectDoesNotExist:
            rel = Relationship(member=emgc.member, client=emgc.client, type=[])

        if _ref_Relationship.EMERGENCY not in rel.type:
            rel.type.append(_ref_Relationship.EMERGENCY)
        rel.nature = emgc.relationship or rel.nature or "??? (migration 0032)"
        rel.save()

    print("All converted. Deleting all member.Referencing and member.EmergencyContact objects...")
    Referencing.objects.all().delete()
    EmergencyContact.objects.all().delete()


def reverse_migrate_relationships(apps, schema_editor):
    """
    Relationship --> Referencing / Emergency Contact
    """
    Relationship = apps.get_model('member', 'Relationship')
    Referencing = apps.get_model('member', 'Referencing')
    EmergencyContact = apps.get_model('member', 'EmergencyContact')

    from souschef.member.models import Relationship as _ref_Relationship  # noqa
    for rel in Relationship.objects.all().order_by('pk'):
        if _ref_Relationship.REFERENT in rel.type:
            print("Reversing member.Relationship #{} to member.Referencing...".format(rel.pk))
            try:
                ref = Referencing.objects.get(referent=rel.member, client=rel.client)
            except Referencing.DoesNotExist:
                ref = Referencing(referent=rel.member, client=rel.client,
                                  referral_reason='??? (reverse migration 0032)',
                                  date=date(1970, 1, 1))

            if isinstance(rel.extra_fields, dict):
                ref.referral_reason = rel.extra_fields.get(
                    'referral_reason', ref.referral_reason)
                ref.date = rel.extra_fields.get('referral_date', ref.date)
            ref.save()

        if _ref_Relationship.EMERGENCY in rel.type:
            print("Reversing member.Relationship #{} to member.EmergencyContact...".format(rel.pk))
            try:
                emgc = EmergencyContact.objects.get(member=rel.member, client=rel.client)
            except EmergencyContact.DoesNotExist:
                emgc = EmergencyContact(member=rel.member, client=rel.client)
            emgc.relationship = rel.nature
            emgc.save()

    print("All reversed. Deleting all member.Relationship objects...")
    Relationship.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0031_client_option_allow_reverse_relation'),
    ]

    operations = [
        # Add Relationship
        migrations.CreateModel(
            name='Relationship',
            options={'verbose_name_plural': 'relationships'},
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nature', models.CharField(max_length=100)),
                ('type', annoying.fields.JSONField(default=[])),
                ('extra_fields', annoying.fields.JSONField(default={})),
                ('remark', models.TextField()),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Client')),
                ('member', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Member')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='relationship',
            unique_together=set([('client', 'member')]),
        ),
        # Migrate
        migrations.RunPython(migrate_relationships, reverse_migrate_relationships),
        # Delete Referencing and EmergencyContact
        migrations.AlterUniqueTogether(
            name='emergencycontact',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='emergencycontact',
            name='client',
        ),
        migrations.RemoveField(
            model_name='emergencycontact',
            name='member',
        ),
        migrations.RemoveField(
            model_name='referencing',
            name='client',
        ),
        migrations.RemoveField(
            model_name='referencing',
            name='referent',
        ),
        migrations.RemoveField(
            model_name='client',
            name='emergency_contacts',
        ),
        migrations.DeleteModel(
            name='EmergencyContact',
        ),
        migrations.DeleteModel(
            name='Referencing',
        ),
    ]



import copy
import collections

from django.db.models import Q, Count, Prefetch
from django.views import generic
from django.utils.translation import string_concat
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.contrib import messages

from souschef.billing.models import (
    Billing, BillingFilter
)
from souschef.order.models import DeliveredOrdersByMonth
from django.urls import reverse_lazy
from souschef.order.models import Order, Order_item
from django.http import HttpResponseRedirect
from souschef.member.models import Client


class BillingList(
        LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    # Display the billing list
    context_object_name = "billings"
    model = Billing
    permission_required = 'sous_chef.read'
    template_name = "billing/list.html"

    def get_context_data(self, **kwargs):
        context = super(BillingList, self).get_context_data(**kwargs)
        uf = BillingFilter(self.request.GET, queryset=self.get_queryset())
        context['filter'] = uf

        return context

    def get_queryset(self):
        uf = BillingFilter(self.request.GET)
        return uf.qs.annotate(Count('orders', distinct=True))


class BillingCreate(
        LoginRequiredMixin, PermissionRequiredMixin, generic.CreateView):
    # View to create the billing
    context_object_name = "billing"
    model = Billing
    permission_required = 'sous_chef.edit'

    def get(self, request):
        date = self.request.GET.get('delivery_date', '-')
        date = date if date != '' else '-'
        year, month = date.split('-')

        if year == '' or month == '':
            messages.add_message(
                self.request, messages.ERROR,
                _("You must select a period to create a billing and click in "
                  "the first step 'Preview'")
            )
            return HttpResponseRedirect(reverse_lazy('billing:add'))

        billing = Billing.objects.billing_create_new(year, month)

        messages.add_message(
            self.request, messages.SUCCESS,
            _("The billing with the identifier #%s "
              "has been successfully created." % billing.id)
        )
        return HttpResponseRedirect(reverse_lazy('billing:list'))


class BillingAdd(
        LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    context_object_name = "orders"
    model = Order
    paginate_by = 20
    permission_required = 'sous_chef.edit'
    template_name = "billing/add.html"

    def get_context_data(self, **kwargs):
        context = super(BillingAdd, self).get_context_data(**kwargs)
        uf = DeliveredOrdersByMonth(
            self.request.GET, queryset=self.get_queryset())
        context['filter'] = uf
        text = ''
        count = 0
        for getVariable in self.request.GET:
            if getVariable == "page":
                continue
            for getValue in self.request.GET.getlist(getVariable):
                if count == 0:
                    text += "?" + getVariable + "=" + getValue
                else:
                    text += "&" + getVariable + "=" + getValue
                count += 1

        text = text + "?" if count == 0 else text + "&"
        context['get'] = text

        return context

    def get_queryset(self):
        uf = DeliveredOrdersByMonth(self.request.GET)
        return uf.qs.select_related(
            'client__member'
        ).only(
            'status',
            'delivery_date',
            'client__member__firstname',
            'client__member__lastname'
        ).prefetch_related(Prefetch(
            'orders',
            queryset=Order_item.objects.all().only(
                'order__id',
                'price',
                'billable_flag'
            )
        ))


class BillingSummaryView(
        LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView):
    # Display summary of billing
    model = Billing
    permission_required = 'sous_chef.read'
    template_name = "billing/view.html"
    context_object_name = "billing"
    queryset = Billing.objects.prefetch_related(Prefetch(
        'orders',
        queryset=Order.objects.all().select_related(
            'client__member'
        ).only(
            'client__member__firstname',
            'client__member__lastname',
            'client__rate_type',
            'client__billing_payment_type'
        ).prefetch_related(Prefetch(
            'orders',
            queryset=Order_item.objects.all().only(
                'order__id',
                'price',
                'billable_flag',
                'size',
                'component_group',
                'total_quantity'
            )
        ))
    ))

    def get_template_names(self):
        if self.request.method == "GET" and \
           self.request.GET.get('print'):
            return ['billing/print_summary.html']
        else:
            return super(BillingSummaryView, self).get_template_names()

    def get_context_data(self, **kwargs):
        context = super(BillingSummaryView, self).get_context_data(**kwargs)
        billing = self.object

        # generate a summary
        zero_statistics = {
            'total_main_dishes': {
                'R': 0,
                'L': 0
            },
            'total_billable_sides': 0,
            'total_amount': 0,
            'total_deliveries': 0,
        }
        # target dict
        summary = copy.deepcopy(zero_statistics)
        summary['payment_types_dict'] = collections.defaultdict(
            lambda: dict(clients=[], **copy.deepcopy(zero_statistics))
        )
        for client, client_summary in billing.summary.items():
            t = client.billing_payment_type
            summary['payment_types_dict'][t]['total_main_dishes']['R'] += (
                client_summary['total_main_dishes']['R']
            )
            summary['payment_types_dict'][t]['total_main_dishes']['L'] += (
                client_summary['total_main_dishes']['L']
            )
            summary['payment_types_dict'][t]['total_billable_sides'] += (
                client_summary['total_billable_sides']
            )
            summary['payment_types_dict'][t]['total_amount'] += (
                client_summary['total_amount']
            )
            nb_deliveries = client.number_of_deliveries_in_month(
                billing.billing_year, billing.billing_month)
            summary['payment_types_dict'][t]['clients'].append({
                'id': client.id,
                'firstname': client.member.firstname,
                'lastname': client.member.lastname,
                'payment_type': client.get_billing_payment_type_display() if (
                    client.billing_payment_type is not None) else '',
                'rate_type': client.get_rate_type_display() if (
                    client.rate_type != 'default') else '',
                'total_main_dishes': client_summary['total_main_dishes'],
                'total_billable_sides': client_summary['total_billable_sides'],
                'total_amount': client_summary['total_amount'],
                'delivery_type_verbose': client.delivery_type_verbose,
                'number_of_deliveries_in_month': nb_deliveries,
            })
            summary['total_main_dishes']['R'] += (
                client_summary['total_main_dishes']['R']
            )
            summary['total_main_dishes']['L'] += (
                client_summary['total_main_dishes']['L']
            )
            summary['total_billable_sides'] += (
                client_summary['total_billable_sides']
            )
            summary['total_amount'] += (
                client_summary['total_amount']
            )
            summary['total_deliveries'] += nb_deliveries

        # sort clients in each payment type group
        for payment_type, statistics in summary['payment_types_dict'].items():
            statistics['clients'].sort(
                key=lambda c: (c['lastname'], c['firstname'])
            )

        # reorder the display for supported & non-supported payment types
        summary['payment_types'] = sorted(
            summary['payment_types_dict'].items(),
            key=lambda tup: {
                ' ': 0,      # 0th position
                'credit': 1,    # 1st position
                'eft': 2,     # 2nd position
                '3rd': 3,  # 3rd position
            }.get(tup[0], 99)  # last position(s)
        )

        context['summary'] = summary

        # Throw a warning if there's any main_dish order with size=None.
        q = Order_item.objects.filter(
            Q(order__in=billing.orders.all()) &
            (Q(size__isnull=True) | Q(size='')) &
            Q(component_group='main_dish')
        )
        if q.exists():
            size_none_orders_info = list(q.values_list(
                'order__id',
                'order__client__member__firstname',
                'order__client__member__lastname'
            ))
            formatted_htmls = ['<ul class="ui list">']
            for i, f, l in size_none_orders_info:
                formatted_htmls.append(
                    '<li><a href="{0}" target="_blank">'
                    '#{1} ({2} {3})'
                    '</a></li>'.format(
                        Order(id=i).get_absolute_url(),
                        i,
                        f,
                        l
                    )
                )
            formatted_htmls.append('</ul>')
            formatted_html = ''.join(formatted_htmls)
            messages.add_message(
                self.request, messages.WARNING,
                string_concat(
                    _('Warning: the order(s) below have not set a "size" '
                      'for main dish and thus have been excluded in '
                      '"Total Main Dishes" column.'),
                    '<br/>',
                    formatted_html
                )
            )
        return context


class BillingOrdersView(
        LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView):
    # Display orders detail of billing
    model = Billing
    permission_required = 'sous_chef.read'
    template_name = "billing/view_orders.html"
    context_object_name = "billing"

    queryset = Billing.objects.prefetch_related(Prefetch(
        'orders',
        queryset=Order.objects.all().select_related(
            'client__member'
        ).only(
            'status',
            'delivery_date',
            'client__member__firstname',
            'client__member__lastname'
        ).prefetch_related(Prefetch(
            'orders',
            queryset=Order_item.objects.all().only(
                'order__id',
                'price',
                'billable_flag'
            )
        ))
    ))

    def get_context_data(self, **kwargs):
        context = super(BillingOrdersView, self).get_context_data(**kwargs)

        if self.request.GET.get('client'):
            # has ?client=client_id
            client_id = int(self.request.GET['client'])
            orders = self.object.orders.filter(client__id=client_id)
            context['orders'] = orders
            context['client'] = Client.objects.get(id=client_id)
        else:
            context['orders'] = self.object.orders.all()

        context['total_amount'] = sum(
            map(lambda o: o.price, context['orders'])
        )
        context['clients'] = list(set(map(
            lambda o: o.client,
            self.object.orders.all()
        )))
        return context


class BillingDelete(
        LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
    model = Billing
    permission_required = 'sous_chef.edit'
    template_name = 'billing_confirm_delete.html'

    def get_success_url(self):
        # 'next' parameter should always be included in POST'ed URL.
        return self.request.GET.get('next') or self.request.POST['next']



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-01-16 15:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0008_auto_20170116_1441'),
    ]

    operations = [
        migrations.AlterField(
            model_name='note',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notes', to='note.NoteCategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='note',
            name='priority',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notes', to='note.NotePriority', verbose_name='Priority'),
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _
from extra_views import InlineFormSet

from souschef.member.models import Client

from souschef.meal.models import COMPONENT_GROUP_CHOICES, COMPONENT_GROUP_CHOICES_SIDES
from souschef.order.models import Order_item, SIZE_CHOICES, OrderStatusChange


class CreateOrderItem(InlineFormSet):
    model = Order_item
    extra = 1
    fields = '__all__'


class UpdateOrderItem(CreateOrderItem):
    extra = 0


class CreateOrdersBatchForm(forms.Form):

    def __init__(self, *args, **kwargs):
        if 'delivery_dates' in kwargs:
            delivery_dates = kwargs['delivery_dates']
            del kwargs['delivery_dates']
        else:
            delivery_dates = None
        super(CreateOrdersBatchForm, self).__init__(*args, **kwargs)

        if delivery_dates:
            for d in delivery_dates:
                self.fields['size_{}'.format(d)] = forms.ChoiceField(
                    choices=SIZE_CHOICES,
                    widget=forms.Select(attrs={'class': ''}),
                    required=True
                )
                self.fields['delivery_{}'.format(d)] = forms.BooleanField(
                    required=False
                )
                self.fields['pickup_{}'.format(d)] = forms.BooleanField(
                    required=False
                )
                self.fields['visit_{}'.format(d)] = forms.BooleanField(
                    required=False
                )

                for meal, meal_translation in COMPONENT_GROUP_CHOICES:
                    if meal is COMPONENT_GROUP_CHOICES_SIDES:
                        continue  # don't put "sides" on the form
                    self.fields[
                        '{}_{}_quantity'.format(meal, d)
                    ] = forms.IntegerField(
                        min_value=0,
                        required=True
                    )

    client = forms.ModelChoiceField(
        required=True,
        label=_('Client'),
        widget=forms.Select(attrs={'class': 'ui search dropdown'}),
        queryset=Client.active.select_related(
            'member'
        ).only(
            'member__firstname',
            'member__lastname'
        ).order_by('member__lastname')
    )

    delivery_dates = forms.CharField(
        required=True,
        label=_('Delivery dates'),
        max_length=200
    )

    override_dates = forms.CharField(
        required=False,
        label=_('Override dates'),
        max_length=200
    )

    is_submit = forms.IntegerField(
        required=True,
        label=_('Is form submit')
    )

    def clean_is_submit(self):
        is_submit = self.cleaned_data['is_submit']
        if is_submit != 1:
            # prevents form submit and force a form refresh
            raise forms.ValidationError(
                _("This field must be 1 to submit the form.")
            )
        return is_submit

    def clean(self):
        """
        For each delivery date, the quantities and options should
        not be all empty. Refs #803.
        """
        cleaned_data = super(CreateOrdersBatchForm, self).clean()
        delivery_dates_str = cleaned_data.get('delivery_dates')
        if delivery_dates_str:
            delivery_dates = delivery_dates_str.split('|')
        else:
            delivery_dates = []
        fields_not_null_check = [
            ('delivery_{}', lambda x: x is True),
            ('pickup_{}', lambda x: x is True),
            ('visit_{}', lambda x: x is True),
        ]
        for meal, meal_translation in COMPONENT_GROUP_CHOICES:
            if meal is COMPONENT_GROUP_CHOICES_SIDES:
                continue  # "sides" not in the form
            else:
                fields_not_null_check.append(
                    ('%s_{}_quantity' % meal, lambda x: x and x > 0))
        for delivery_date in delivery_dates:
            if all([
                    bool(check_fn(
                        cleaned_data.get(field_template.format(delivery_date))
                    )) is False
                    for field_template, check_fn in fields_not_null_check
            ]):
                # Error-highlight all fields on that date.
                for field_template, check_fn in fields_not_null_check:
                    self.add_error(
                        field_template.format(delivery_date),
                        forms.ValidationError(
                            _("Empty order is not allowed."),
                            code='empty_delivery_date'
                        )
                    )

        return cleaned_data


class OrderStatusChangeForm(forms.ModelForm):

    class Meta:
        model = OrderStatusChange
        fields = [
            'order', 'status_from', 'status_to', 'reason'
        ]
        widgets = {
            'status_to': forms.Select(
                attrs={'class': 'ui status_to dropdown disabled'}
            ),
            'reason': forms.Textarea(attrs={'rows': 2}),
        }

    def clean(self):
        cleaned_data = super(OrderStatusChangeForm, self).clean()
        status_to = cleaned_data.get('status_to')
        reason = cleaned_data.get('reason')

        if status_to == 'N' and not reason:  # No Charge without reason
            self.add_error(
                'reason',
                forms.ValidationError(
                    _('A reason is required for No Charge order.'),
                    code='no_charge_reason_required'
                )
            )
        return cleaned_data



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.order.views import (
    CancelOrder,
    CreateDeleteOrderClientBill,
    CreateOrder,
    CreateOrdersBatch,
    DeleteOrder,
    OrderDetail,
    OrderList,
    UpdateOrder,
    UpdateOrderStatus,
)

app_name = "order"

urlpatterns = [
    url(_(r'^list/$'), OrderList.as_view(), name='list'),
    url(_(r'^view/(?P<pk>\d+)/$'), OrderDetail.as_view(), name='view'),
    url(_(r'^create/$'), CreateOrder.as_view(), name='create'),
    # Multiple orders as once
    url(_(r'^create/batch$'),
        CreateOrdersBatch.as_view(), name='create_batch'),

    url(_(r'^update/(?P<pk>\d+)/$'), UpdateOrder.as_view(), name='update'),
    url(
        _(r'^update/(?P<pk>\d+)/status$'),
        UpdateOrderStatus.as_view(),
        name='update_status'
    ),
    url(
        _(r'^update/(?P<pk>\d+)/client_bill$'),
        CreateDeleteOrderClientBill.as_view(),
        name='update_client_bill'
    ),
    url(_(r'^cancel/(?P<pk>\d+)/$'), CancelOrder.as_view(), name='cancel'),
    url(_(r'^delete/(?P<pk>\d+)/$'), DeleteOrder.as_view(), name='delete'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-23 01:15
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0004_fix004a'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='route',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='member.Route', verbose_name='route'),
        ),
    ]



import csv

from django.core.management.base import BaseCommand
from souschef.member.models import Client, Member
from souschef.member.models import Option, Client_option


class Command(BaseCommand):
    help = 'Data: import clients relationships from given csv file.'

    ROW_MID = 0
    ROW_MON = 1
    ROW_TUE = 2
    ROW_WED = 3
    ROW_THU = 4
    ROW_FRI = 5
    ROW_SAT = 6
    ROW_FOOD_PREP_PUREE = 7
    ROW_FOOD_PREP_CUT = 8
    ROW_MLABEL = 9
    ROW_ING_BEEF = 10

    def add_arguments(self, parser):
        parser.add_argument(
            '--file',
            default=False,
            help='Import mock data instead of actual data',
        )

    def handle(self, *args, **options):
        if options['file']:
            file = 'mock_meals.csv'
        else:
            file = 'clients_meals.csv'

        food_prep_puree = Option.objects.get(
            name='Puree all'
        )
        food_prep_cut = Option.objects.get(
            name='Cut up meat'
        )

        with open(file) as f:
            reader = csv.reader(f, delimiter=';')
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    client = Client.objects.get(member=member)
                    days = [
                        self.ROW_MON,
                        self.ROW_TUE,
                        self.ROW_WED,
                        self.ROW_THU,
                        self.ROW_FRI,
                        self.ROW_SAT
                    ]
                    meals_schedule = []
                    prefs = {}

                    for day in days:
                        if row[day] != "":
                            meals_schedule.append(row[day])
                            delivery_day = day - 1
                            # Hack: there is no column Thursday in the csv file
                            if delivery_day > 3:
                                delivery_day -= 1

                            prefs['size_' + row[day]] = \
                                row[11 + (delivery_day * 10) + 1]
                            prefs['main_dish_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 0])
                            prefs['dessert_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 5])
                            prefs['fruit_salad_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 2])
                            prefs['green_salad_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 3])
                            prefs['pudding_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 6])
                            prefs['diabetic_' + row[day] + '_quantity'] = \
                                int(row[11 + (delivery_day * 10) + 4])

                    client.set_simple_meals_schedule(meals_schedule)
                    client.meal_default_week = prefs
                    client.save()

                    # Food preparation
                    if row[self.ROW_FOOD_PREP_CUT] == '1':
                        Client_option.objects.create(
                            client=client,
                            option=food_prep_cut
                        )
                    if row[self.ROW_FOOD_PREP_PUREE] == '1':
                        Client_option.objects.create(
                            client=client,
                            option=food_prep_puree
                        )

                except Member.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing member'))
                except Client.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing client'))



from django.core.management.commands import makemessages


class Command(makemessages.Command):
    """
    Support argument --add-location of GNU gettext utilities.

    A feature request has been submitted to Django at the time of writing
    this module (Django 1.11rc1). If a future Django version supports this
    argument, we can then safely remove this module.

    Ref: https://groups.google.com/forum/#!topic/django-developers/IkrjfBDA7iE
    """

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            '--add-location',
            action='store',
            default='full',
            type=str,
            choices=['full', 'file', 'never'],
            help='If full, generates the location lines with both file name '
                 'and line number. If file, the line number part is omitted. '
                 'If never, completely suppresses the lines '
                 '(same as --no-location).',
            dest='add_location',
        )

    def handle(self, *args, **options):
        add_location = options.pop('add_location')
        arg_str = "--add-location={}".format(add_location)
        self.msgmerge_options = (
            makemessages.Command.msgmerge_options[:] + [arg_str]
        )
        self.msguniq_options = (
            makemessages.Command.msguniq_options[:] + [arg_str]
        )
        self.msgattrib_options = (
            makemessages.Command.msgattrib_options[:] + [arg_str]
        )
        self.xgettext_options = (
            makemessages.Command.xgettext_options[:] + [arg_str]
        )

        super(Command, self).handle(*args, **options)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-19 20:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0002_auto_20160818_2104'),
    ]

    operations = [
        migrations.AlterField(
            model_name='note',
            name='date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date'),
        ),
    ]



from django.conf.urls import url
from souschef.billing.views import (
    BillingList, BillingCreate, BillingDelete,
    BillingSummaryView, BillingOrdersView, BillingAdd
)
from django.utils.translation import ugettext_lazy as _

app_name = "billing"

urlpatterns = [
    url(_(r'^list/$'), BillingList.as_view(), name="list"),
    url(_(r'^create/$'), BillingCreate.as_view(), name="create"),
    url(_(r'^add/$'), BillingAdd.as_view(), name="add"),
    url(_(r'^view/(?P<pk>\d+)/$'),
        BillingSummaryView.as_view(), name="view"),
    url(_(r'^view/(?P<pk>\d+)/orders/$'),
        BillingOrdersView.as_view(), name="view_orders"),
    url(_(r'^delete/(?P<pk>\d+)/$'), BillingDelete.as_view(), name='delete'),
]



"""sous-chef URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/

Examples:

Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')

Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')

Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))

"""
from django.conf.urls import include, url
from django.contrib import admin


from souschef.page.views import HomeView
from django.conf import settings
from django.conf.urls.static import static
from django.conf.urls.i18n import i18n_patterns
from django.utils.translation import ugettext_lazy as _

urlpatterns = i18n_patterns(
    url(
        _(r'^admin/'),
        admin.site.urls
    ),
    url(
        _(r'^meal/'),
        include('souschef.meal.urls', namespace="meal")
    ),
    url(
        _(r'^member/'),
        include('souschef.member.urls', namespace="member")
    ),
    url(
        _(r'^notification/'),
        include('souschef.notification.urls', namespace="notification")
    ),
    url(
        _(r'^order/'),
        include('souschef.order.urls', namespace="order")
    ),
    url(
        r'^p/',
        include('souschef.page.urls', namespace="page")
    ),
    url(r'^$', HomeView.as_view(), name='home'),
    url(
        _(r'^delivery/'),
        include('souschef.delivery.urls', namespace="delivery")
    ),
    url(
        _(r'^note/'),
        include('souschef.note.urls', namespace="note")
    ),
    url(
        _(r'^billing/'),
        include('souschef.billing.urls', namespace="billing")
    ),
    url(
        _(r'^avatar/'),
        include('avatar.urls')
        ),
    prefix_default_language=False
) + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from django.core.management.base import BaseCommand
from souschef.order.factories import OrderFactory


class Command(BaseCommand):
    help = 'Creates a happy bunch of clients with orders'

    def handle(self, *args, **options):
        orders = OrderFactory.create_batch(500, status='D')
        self.stdout.write(
            self.style.SUCCESS(
                'Successfully created client {} orders'.format(orders)
            )
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-17 16:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0011_client_delivery_note'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClientScheduledStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status_from', models.CharField(choices=[('D', 'Pending'), ('A', 'Active'), ('S', 'Paused'), ('N', 'Stop: no contact'), ('C', 'Stop: contact'), ('I', 'Deceased')], max_length=1)),
                ('status_to', models.CharField(choices=[('D', 'Pending'), ('A', 'Active'), ('S', 'Paused'), ('N', 'Stop: no contact'), ('C', 'Stop: contact'), ('I', 'Deceased')], max_length=1)),
                ('reason', models.CharField(blank=True, default='', max_length=200)),
                ('change_date', models.DateField(blank=True, default=django.utils.timezone.now, null=True)),
                ('change_state', models.CharField(choices=[('ALONE', 'Alone'), ('START', 'Start'), ('END', 'End')], default='ALONE', max_length=5)),
                ('operation_status', models.CharField(choices=[('NEW', 'To be processed'), ('PRO', 'Processed'), ('ERR', 'Error')], default='NEW', max_length=3)),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Client')),
                ('linked_scheduled_status', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='member.ClientScheduledStatus')),
            ],
        ),
    ]



from django.contrib import admin

from souschef.member.models import (
    Member, Client, Contact, Address,
    Route, Option, Relationship,
    DeliveryHistory, ClientScheduledStatus
)


class IngredientsToAvoidInline(admin.TabularInline):
    model = Client.ingredients_to_avoid.through


class ComponentsToAvoidInline(admin.TabularInline):
    model = Client.components_to_avoid.through


class RestrictionsInline(admin.TabularInline):
    model = Client.restrictions.through


class OptionsInline(admin.TabularInline):
    model = Client.options.through


class ContactInline(admin.TabularInline):
    model = Contact


class RelationshipInline(admin.TabularInline):
    model = Relationship
    extra = 0


class MemberAdmin(admin.ModelAdmin):
    search_fields = ['lastname', 'firstname']
    list_display = ('full_name', 'address', 'work_information', 'updated_at')
    inlines = [
        ContactInline
    ]

    def full_name(self, obj):
        return ("%s %s" % (obj.firstname, obj.lastname))
    full_name.short_description = 'Name'


class ClientAdmin(admin.ModelAdmin):
    search_fields = ['member__lastname', 'member__firstname']
    list_filter = ('status', 'route', 'delivery_type')
    list_display = (
        'member',
        'status',
        'language',
        'delivery_type',
        'gender',
        'route')
    inlines = [
        RelationshipInline,
        OptionsInline,
        RestrictionsInline,
        ComponentsToAvoidInline,
        IngredientsToAvoidInline
    ]


class ContactAdmin(admin.ModelAdmin):
    search_fields = ['member__lastname', 'member__firstname']
    list_display = ('member', 'type', 'value', )
    list_filter = ('type',)


class RelationshipAdmin(admin.ModelAdmin):
    search_fields = ['member__lastname', 'member__firstname',
                     'client__member__lastname', 'client__member__firstname',
                     'type', 'nature', 'extra_fields']
    list_display = ('__str__', 'member', 'type', 'nature', 'client',
                    'extra_fields')


admin.site.register(Member, MemberAdmin)
admin.site.register(Client, ClientAdmin)
admin.site.register(ClientScheduledStatus)
admin.site.register(Route)
admin.site.register(DeliveryHistory)
admin.site.register(Contact, ContactAdmin)
admin.site.register(Address)
admin.site.register(Relationship, RelationshipAdmin)
admin.site.register(Option)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-02-19 16:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0035_auto_20210115_1155'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='address',
            options={'ordering': ['number', 'street', 'apartment'], 'verbose_name_plural': 'addresses'},
        ),
        migrations.AlterField(
            model_name='client',
            name='billing_payment_type',
            field=models.CharField(blank=True, choices=[(' ', '----'), ('3rd', '3rd Party'), ('cash', 'Cash'), ('cheque', 'Cheque'), ('credit', 'Credit card'), ('eft', 'EFT'), ('etransfert', 'e-Transfer')], max_length=10, null=True, verbose_name='Payment Type'),
        ),
        migrations.AlterField(
            model_name='member',
            name='address',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='member.Address', verbose_name='address'),
        ),
    ]



from django.contrib import admin
from django.db.models.functions import Lower
from souschef.meal.models import (Component, Restricted_item,
                         Ingredient, Component_ingredient,
                         Incompatibility, Menu,
                         COMPONENT_GROUP_CHOICES,
                         INGREDIENT_GROUP_CHOICES,
                         RESTRICTED_ITEM_GROUP_CHOICES)


class ComponentsInline(admin.TabularInline):
    model = Menu.components.through


class ComponentIngredientInline(admin.TabularInline):
    model = Component_ingredient


class IncompatibilityInline(admin.TabularInline):
    model = Incompatibility


class MenuAdmin(admin.ModelAdmin):
    """Allows accessing menu components within the Menu admin."""
    inlines = [
        ComponentsInline
    ]


class ComponentAdmin(admin.ModelAdmin):
    """Allows accessing ingredients within the Component admin."""
    inlines = [
        ComponentIngredientInline
    ]
    list_display = ('name', 'component_group')
    search_fields = ('name',)

    def get_ordering(self, request):
        return [Lower('name')]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term)
        search_list = [choice[0] for
                       choice in COMPONENT_GROUP_CHOICES
                       if search_term.lower() in choice[1].lower()]
        if search_list:
            queryset |= self.model.objects.filter(
                component_group__in=search_list)
        return queryset, use_distinct


class IngredientAdmin(admin.ModelAdmin):
    """Allows more control over the display of ingredients in the admin."""
    list_display = ('name', 'ingredient_group',)
    search_fields = ('name',)

    def get_ordering(self, request):
        return [Lower('name')]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term)
        search_list = [choice[0] for
                       choice in INGREDIENT_GROUP_CHOICES
                       if search_term.lower() in choice[1].lower()]
        if search_list:
            queryset |= self.model.objects.filter(
                ingredient_group__in=search_list)
        return queryset, use_distinct


class Restricted_itemAdmin(admin.ModelAdmin):
    """Allows accessing ingredients within the Restricted_item admin."""
    inlines = [
        IncompatibilityInline
    ]
    list_display = ('name', 'restricted_item_group',)
    search_fields = ('name',)

    def get_ordering(self, request):
        return [Lower('name')]

    def get_search_results(self, request, queryset, search_term):
        # search for group choices in their display value instead of DB value
        queryset, use_distinct = super().get_search_results(
            request, queryset, search_term)
        search_list = [choice[0] for
                       choice in RESTRICTED_ITEM_GROUP_CHOICES
                       if search_term.lower() in choice[1].lower()]
        if search_list:
            queryset |= self.model.objects.filter(
                restricted_item_group__in=search_list)
        return queryset, use_distinct

admin.site.register(Component, ComponentAdmin)
admin.site.register(Restricted_item, Restricted_itemAdmin)
admin.site.register(Ingredient, IngredientAdmin)
admin.site.register(Menu, MenuAdmin)



# -*- coding: utf-8 -*-
import pkg_resources

from django.conf import settings
from souschef.member.models import Client, Route
from souschef.order.models import Order, ORDER_STATUS_ORDERED
from souschef.note.models import Note, NoteFilter
import datetime


def get_sous_chef_version():
    return pkg_resources.require("souschef")[0].version


def total(request):
    """ Passing entities total into RequestContext. """

    clients = Client.objects.count()
    orders = Order.objects.count()
    routes = Route.objects.count()
    # Only unread messages
    notes = Note.unread.count()

    COMMON_CONTEXT = {
        'CLIENTS_TOTAL': clients,
        'ORDERS_TOTAL': orders,
        'ROUTES_TOTAL': routes,
        'NOTES_TOTAL': notes,
        # This is pretty bad separation of concert to puth the following here
        # But since there is no actual view associated with menu.html (it's
        # just included is base.html, this is the most DRY way I found
        'CLIENT_FILTER_DEFAULT_STATUS': Client.ACTIVE,
        'ORDER_FILTER_DEFAULT_STATUS': ORDER_STATUS_ORDERED,
        'ORDER_FILTER_DEFAULT_DATE': datetime.datetime.now(),
        'NOTE_FILTER_DEFAULT_IS_READ': NoteFilter.NOTE_STATUS_UNREAD,
        'SC_ENVIRONMENT_NAME': settings.SOUSCHEF_ENVIRONMENT_NAME,
        'SC_VERSION': get_sous_chef_version(),
        'GIT_HEAD': settings.GIT_HEAD,
        'GIT_TAG': settings.GIT_TAG,
    }

    return COMMON_CONTEXT



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-25 23:37
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0006_auto_20160803_2217'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='order_item',
            name='component',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-12 15:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0015_route_client_id_sequence'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='gender',
            field=models.CharField(blank=True, choices=[('F', 'Female'), ('M', 'Male')], default='U', max_length=1, null='True'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-11-22 04:58
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0019_auto_20161111_1750'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='client',
            options={'ordering': ['-member__created_at'], 'verbose_name_plural': 'clients'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-08-03 11:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0004_fix004a'),
    ]

    operations = [
        migrations.AddField(
            model_name='order_item',
            name='component_group',
            field=models.CharField(choices=[('main_dish', 'Main Dish'), ('dessert', 'Dessert'), ('diabetic', 'Diabetic Dessert'), ('fruit_salad', 'Fruit Salad'), ('green_salad', 'Green Salad'), ('pudding', 'Pudding'), ('compote', 'Compote')], max_length=100, null=True, verbose_name='component group'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-18 21:04
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='note',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-23 01:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0002_ingredient_ingredient_group'),
        ('member', '0003_auto_20160615_2100'),
    ]

    operations = [
        migrations.CreateModel(
            name='Client_avoid_component',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='Client_avoid_ingredient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.AddField(
            model_name='client_option',
            name='value',
            field=models.CharField(max_length=100, null=True, verbose_name='value'),
        ),
        migrations.AlterField(
            model_name='client',
            name='billing_payment_type',
            field=models.CharField(choices=[('check', 'Check'), ('cash', 'Cash'), ('debit', 'Debit card'), ('credit', 'Credit card'), ('eft', 'EFT')], max_length=10, null=True, verbose_name='Payment Type'),
        ),
        migrations.AlterField(
            model_name='client',
            name='gender',
            field=models.CharField(blank=True, choices=[('', 'Gender'), ('F', 'Female'), ('M', 'Male')], default='U', max_length=1, null='True'),
        ),
        migrations.AlterField(
            model_name='client',
            name='route',
            field=models.ForeignKey(blank=True, default='', on_delete=django.db.models.deletion.CASCADE, to='member.Route', verbose_name='route'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='option',
            name='option_group',
            field=models.CharField(choices=[('main dish size', 'Main dish size'), ('dish', 'Dish'), ('preparation', 'Preparation'), ('other order item', 'Other order item')], max_length=100, verbose_name='option group'),
        ),
        migrations.AddField(
            model_name='client_avoid_ingredient',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='member.Client', verbose_name='client'),
        ),
        migrations.AddField(
            model_name='client_avoid_ingredient',
            name='ingredient',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Ingredient', verbose_name='ingredient'),
        ),
        migrations.AddField(
            model_name='client_avoid_component',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='member.Client', verbose_name='client'),
        ),
        migrations.AddField(
            model_name='client_avoid_component',
            name='component',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Component', verbose_name='component'),
        ),
    ]



from django import template

register = template.Library()


@register.filter(name='get_item')
def get_item(o, key):
    try:
        return o[key]
    except Exception:
        return None


@register.filter(name='alter_field_class')
def alter_field_class(field, css):
    return field.as_widget(attrs={"class": css})



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-21 11:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0003_fix224a'),
    ]

    operations = [
        migrations.AddField(
            model_name='component_ingredient',
            name='date',
            field=models.DateField(blank=True, null=True, verbose_name='date'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-01-15 04:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def migrate_old_data(apps, schema_editor):
    # set for each product the correct language
    NotePriority = apps.get_model('note', 'NotePriority')
    Note = apps.get_model('note', 'Note')

    normal_priority = NotePriority.objects.get(name="Normal")
    urgent_priority = NotePriority.objects.get(name="Urgent")

    for note in Note.objects.all():
        if note.priority == normal_priority.name.lower():
            note.priority_temp = normal_priority
            note.save()
        elif note.priority == urgent_priority.name.lower():
            note.priority_temp = urgent_priority
            note.save()


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0004_notepriority'),
    ]

    operations = [
        migrations.AddField(
            model_name='note',
            name='priority_temp',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notes', to='note.NotePriority', verbose_name='Priority'),
        ),
        migrations.RunPython(migrate_old_data),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-11-22 04:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='billing',
            name='total_amount',
            field=models.DecimalField(decimal_places=2, max_digits=8, verbose_name='total_amount'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-07-23 15:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0013_date_modified'),
    ]

    operations = [
        migrations.AlterField(
            model_name='note',
            name='date_created',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date Created'),
        ),
    ]



import re
from django import forms
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from localflavor.ca.forms import CAPhoneNumberField


class CAPhoneNumberExtField(CAPhoneNumberField):
    """Canadian phone number form field."""
    phone_digits_re = re.compile(
            r'^(?:1-?)?(\d{3})[-\.]?(\d{3})[-\.]?(\d{4})$'
    )
    phone_digits_with_ext = re.compile(
            r'^(?:1-?)?(\d{3})[-\.]?(\d{3})[-\.]?(\d{4})#?(\d*)$'
        )

    default_error_messages = {
        'invalid':
        _('Phone numbers must be in XXX-XXX-XXXX # XXXXX format.'),
    }

    def clean(self, value):
        try:
            return super(CAPhoneNumberExtField, self).clean(value)
        except forms.ValidationError as error:
            value = re.sub('(\(|\)|\s+)', '', smart_text(value))
            m = self.phone_digits_with_ext.search(value)
            if m:
                return '%s-%s-%s #%s' % (m.group(1), m.group(2),
                                         m.group(3), m.group(4))
            raise error



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-26 00:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0005_fix241b'),
    ]

    operations = [
        migrations.AddField(
            model_name='menu',
            name='components',
            field=models.ManyToManyField(through='meal.Menu_component', to='meal.Component'),
        ),
        migrations.AddField(
            model_name='restricted_item',
            name='ingredients',
            field=models.ManyToManyField(through='meal.Incompatibility', to='meal.Ingredient'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-05 17:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0010_auto_20160803_2052'),
    ]

    operations = [
        migrations.AddField(
            model_name='client',
            name='delivery_note',
            field=models.TextField(blank=True, null=True, verbose_name='Delivery Note'),
        ),
    ]



import datetime
import json

from django.db.models import Q
from django.test import RequestFactory
from django.test import TestCase
from django.contrib.auth.models import User
from django.urls import reverse_lazy, reverse
from django.utils import timezone as tz
from django.utils.translation import ugettext_lazy, ugettext

from souschef.meal.models import (Menu, Component, Component_ingredient, Ingredient,
                         COMPONENT_GROUP_CHOICES_SIDES)
from souschef.meal.factories import (IngredientFactory, ComponentFactory,
                            ComponentIngredientFactory,
                            IncompatibilityFactory, RestrictedItemFactory)
from souschef.order.models import Order
from souschef.member.models import (Client, Member, Route, Restriction, DAYS_OF_WEEK,
                           Client_avoid_ingredient, DeliveryHistory)
from souschef.member.factories import (AddressFactory, MemberFactory, ClientFactory,
                              RouteFactory, DeliveryHistoryFactory)
from souschef.sous_chef.tests import TestMixin as SousChefTestMixin

from .filters import KitchenCountOrderFilter


class KitchenCountReportTestCase(SousChefTestMixin, TestCase):

    # This data set includes 'Ground porc' clashing ingredient
    # This data set includes 'Tracy' client lastname
    #   and his side dish is 'compote'
    fixtures = ['sample_data']

    @classmethod
    def setUpTestData(cls):
        Menu.create_menu_and_components(
            datetime.date(2016, 5, 21),
            ['Ginger pork',
             'Green Salad', 'Fruit Salad',
             'Day s Dessert', 'Day s Diabetic Dessert',
             'Day s Pudding', 'Day s Compote'])

    def setUp(self):
        self.force_login()

    def test_clashing_ingredient(self):
        """An ingredient we know will clash must be in the page"""
        self.today = datetime.date.today()
        # create orders today
        clients = Client.active.all()
        Order.objects.auto_create_orders(
            self.today, clients)
        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name='Ginger pork')
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(
            main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(
                component=main_dish,
                ingredient=ing,
                date=self.today)
            ci.save()
        # Add extra ingredient so that more clashing clients will
        #  require two pages on the PDF kitchen count report
        extra = Ingredient.objects.get(name='Walnuts')
        ci = Component_ingredient(
            component=main_dish,
            ingredient=extra,
            date=self.today)
        ci.save()
        # add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES)
        extra = Ingredient.objects.get(name='Cabbage')
        ci = Component_ingredient(
            component=sides_component,
            ingredient=extra,
            date=self.today)
        ci.save()
        # menu today
        Menu.create_menu_and_components(
            self.today,
            ['Ginger pork',
             'Green Salad', 'Fruit Salad',
             'Day s Dessert', 'Day s Diabetic Dessert',
             'Day s Pudding', 'Day s Compote'])
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Ground porc' in response.content)

    def test_no_components(self):
        """No orders on this day therefore no component summary"""
        response = self.client.get(reverse_lazy(
            'delivery:kitchen_count_date',
            kwargs={'year': '2015', 'month': '05', 'day': '21'}
        ))
        response = self.client.get('/delivery/kitchen_count/2015/05/21/')
        self.assertTrue(b'SUBTOTAL' not in response.content)

    def test_labels_show_restrictions(self):
        """An ingredient we know will clash must be in the labels"""
        # generate orders today
        self.today = datetime.date.today()
        clients = Client.active.all()
        Order.objects.auto_create_orders(
            self.today, clients)
        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name='Ginger pork')
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(
            main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(
                component=main_dish,
                ingredient=ing,
                date=self.today)
            ci.save()
        # Add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES)
        # This ingredient is in the restrictions of some clients in the data
        sides_ingredient = Ingredient.objects.get(name='Brussel sprouts')
        ci = Component_ingredient(
            component=sides_component,
            ingredient=sides_ingredient,
            date=self.today)
        ci.save()
        # menu today
        Menu.create_menu_and_components(
            self.today,
            ['Ginger pork',
             'Green Salad', 'Fruit Salad',
             'Day s Dessert', 'Day s Diabetic Dessert',
             'Day s Pudding', 'Day s Compote', 'Days Sides'])

        self.client.get(reverse_lazy('delivery:kitchen_count'))
        response = self.client.get(reverse_lazy('delivery:mealLabels'))
        self.assertTrue('ReportLab' in repr(response.content))

    def test_pdf_report_show_restrictions(self):
        """An ingredient we know will clash must be in the pdf report"""
        # generate orders today
        self.today = datetime.date.today()
        clients = Client.active.all()
        Order.objects.auto_create_orders(
            self.today, clients)
        Menu.create_menu_and_components(
            self.today,
            ['Ginger pork',
             'Green Salad', 'Fruit Salad',
             'Day s Dessert', 'Day s Diabetic Dessert',
             'Day s Pudding', 'Day s Compote'])

        self.client.get('/delivery/kitchen_count/')
        response = self.client.get('/delivery/viewDownloadKitchenCount/')
        self.assertTrue('ReportLab' in repr(response.content))


class ChooseDayMainDishIngredientsTestCase(SousChefTestMixin, TestCase):

    fixtures = ['sample_data']

    @classmethod
    def setUpTestData(cls):
        # This data set includes 'Ginger pork' main dish
        # This data set includes 'Coq au vin' main dish
        # This data set includes 'Ground porc' main dish ingredient
        # This data set includes 'Pepper' available ingredient
        # create orders for today
        clients = Client.active.all()
        Order.objects.auto_create_orders(
            datetime.date.today(),
            clients)

    def setUp(self):
        self.force_login()

    def test_known_ingredients(self):
        """Two ingredients we know must be in the page"""
        response = self.client.get(reverse_lazy('delivery:meal'))
        self.assertTrue(b'Ground porc' in response.content and
                        b'Pepper' in response.content)

    def test_date_with_dish_next(self):
        """From ingredient choice go to Kitchen Count Report."""
        response = self.client.get(reverse_lazy('delivery:meal'))
        maindish = Component.objects.get(name='Ginger pork')
        cis = Component_ingredient.objects.filter(
            date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        req = {}
        req['_update'] = 'Confirm all ingredients'
        req['maindish'] = str(maindish.id)
        req['ingredients'] = ing_ids
        req['sides_ingredients'] = [
            Ingredient.objects.filter(name='zucchini').first().id]
        response = self.client.post(reverse_lazy('delivery:meal'), req)
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Ginger pork' in response.content)

    def test_remember_day_ingredients(self):
        """After Kitchen Count Report we remember chosen ingredients."""
        response = self.client.get(reverse_lazy('delivery:meal'))
        maindish = Component.objects.get(name='Ginger pork')
        cis = Component_ingredient.objects.filter(
            date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        req = {}
        req['_update'] = 'Confirm all ingredients'
        req['maindish'] = str(maindish.id)
        req['ingredients'] = ing_ids
        req['sides_ingredients'] = [
            Ingredient.objects.filter(name='Onions').first().id]
        response = self.client.post(reverse_lazy('delivery:meal'), req)
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Ginger pork' in response.content)
        response = self.client.get(reverse_lazy('delivery:meal'))
        self.assertTrue(b'Ginger pork' in response.content)

    def test_restore_dish_recipe(self):
        """Restore dish ingredients to those of recipe."""
        # dish : Ginger pork with added Pepper
        response = self.client.get(reverse_lazy('delivery:meal'))
        maindish = Component.objects.get(name='Ginger pork')
        cis = Component_ingredient.objects.filter(
            date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]
        # dish and added ingredient must be in kitchen count
        req = {}
        req['_update'] = 'Confirm all ingredients'
        req['maindish'] = str(maindish.id)
        req['ingredients'] = ing_ids + \
            [Ingredient.objects.get(name='Pepper').id]
        req['sides_ingredients'] = [Ingredient.objects.all().first().id]
        response = self.client.post(reverse_lazy('delivery:meal'), req)
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Ginger pork' in response.content and
                        b'Pepper' in response.content)
        # restore recipe
        response = self.client.post(
            reverse_lazy('delivery:meal'),
            {'_restore': 'Restore recipe'}
        )
        # update ingredients
        req = {}
        req['_update'] = 'Confirm all ingredients'
        req['maindish'] = str(maindish.id)
        req['ingredients'] = ing_ids
        req['sides_ingredients'] = [Ingredient.objects.all().first().id]
        response = self.client.post(reverse_lazy('delivery:meal'), req)
        # check that we have Ginger pork with no Pepper in Kitchen count
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Ginger pork' in response.content and
                        b'Pepper' not in response.content)

    def test_change_main_dish(self):
        """Change dish then go directly to Kitchen Count Report."""
        maindish = Component.objects.get(name='Coq au vin')
        cis = Component_ingredient.objects.filter(
            date=None, component=maindish)
        ing_ids = [ci.ingredient.id for ci in cis]

        response = self.client.get(
            reverse_lazy('delivery:meal_id', args=[maindish.id]))
        req = {}
        req['_update'] = 'Update ingredients'
        req['maindish'] = str(maindish.id)
        req['ingredients'] = ing_ids
        req['sides_ingredients'] = [Ingredient.objects.all().first().id]
        response = self.client.post(reverse_lazy('delivery:meal'), req)
        response = self.client.get(reverse_lazy('delivery:kitchen_count'))
        self.assertTrue(b'Coq au vin' in response.content)

    def test_post_invalid_form(self):
        """Invalid form."""
        response = self.client.get(reverse_lazy('delivery:meal'))
        req = {}
        req['_update'] = 'Next: Print Kitchen Count'
        req['maindish'] = 'wrong'
        response = self.client.post(reverse_lazy('delivery:meal'), req)

        self.assertIn(
            ugettext(
                'Select a valid choice. That choice is not one of '
                'the available choices.').encode(response.charset),
            response.content
        )

    def test_if_valid_no_red_sign(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        response = self.client.get(reverse('delivery:order'))
        self.assertNotIn(b'<i class="warning sign red icon"></i>',
                         response.content)
        response = self.client.get(reverse('delivery:refresh_orders'))
        self.assertNotIn(b'<i class="warning sign red icon"></i>',
                         response.content)

    def test_no_route_error(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        c = o.client
        c.route = None
        c.save()
        response = self.client.get(reverse('delivery:order'))
        self.assertIn(b'<i class="warning sign red icon"></i>',
                      response.content)
        response = self.client.get(reverse('delivery:refresh_orders'))
        self.assertIn(b'<i class="warning sign red icon"></i>',
                      response.content)

    def test_not_geolocalized_error(self):
        o = Order.objects.first()
        o.delivery_date = tz.now()
        o.save()
        c = o.client
        c.member.address.latitude = None
        c.member.address.longitude = None
        c.member.address.save()
        response = self.client.get(reverse('delivery:order'))
        self.assertIn(b'<i class="warning sign red icon"></i>',
                      response.content)
        response = self.client.get(reverse('delivery:refresh_orders'))
        self.assertIn(b'<i class="warning sign red icon"></i>',
                      response.content)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:meal')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:meal')
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)

    def test_disable_form_fields_for_users_that_do_not_have_edit_permission(self):  # noqa
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:meal')
        # Run
        response = self.client.get(url)
        # Check
        for field in response.context['form']:
            assert field.field.disabled

    def test_do_not_allow_users_that_do_not_have_edit_permission_to_post_data(self):  # noqa
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:meal')
        # Run
        response = self.client.post(url)
        # Check
        self.assertEqual(response.status_code, 403)


class DeliveryRouteSheetTestCase(SousChefTestMixin, TestCase):

    # This data set includes 'Blondin' client lastname
    #   in 'Centre Sud' route
    # This data set includes 'Tracy' client lastname
    #   in 'Mile-End' route
    #   and his side dish is 'compote'
    fixtures = ['sample_data']

    def setUp(self):
        # generate orders today
        self.today = datetime.date.today()
        clients = Client.active.all()
        Order.objects.auto_create_orders(
            self.today, clients)
        self.route_id = Route.objects.get(name='Centre Sud').id
        self.force_login()

    def test_query(self):
        """Sample route sheet query."""
        route_list = Order.get_delivery_list(self.today, self.route_id)
        self.assertTrue('Blondin' in repr(route_list))

    def test_extra_similar_sides(self):
        """Test cumulative quantities for similar side dishes."""
        # Add two separate extra compote order items for 'Tracy'
        member = Member.objects.filter(lastname='Tracy')[0]
        client = Client.objects.get(member=member.id)
        order = Order.objects.get(client=client.id, delivery_date=self.today)
        self.assertEqual(order.client.id, client.id)
        mile_end_id = Route.objects.get(name='Mile-End').id
        route_list = Order.get_delivery_list(self.today, mile_end_id)
        self.assertTrue('Tracy' in repr(route_list))

    def test_sheet(self):
        """Sample route sheet page."""
        client = Client.objects.filter(member__lastname='Blondin')[0]
        route = Route.objects.get(pk=self.route_id)
        DeliveryHistoryFactory(
            route=route,
            date=tz.datetime.today(),
            client_id_sequence=[client.id]
        )
        response = self.client.get(
            reverse_lazy('delivery:route_sheet', args=[self.route_id]))
        self.assertTrue(b'Blondin' in response.content)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:route_sheet', args=[self.route_id])
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        DeliveryHistoryFactory(
            route=Route.objects.get(pk=self.route_id),
            date=tz.datetime.today()
        )
        url = reverse('delivery:route_sheet', args=[self.route_id])
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class RedirectAnonymousUserTestCase(SousChefTestMixin, TestCase):

    fixtures = ['sample_data']

    def test_anonymous_user_gets_redirect_to_login_page(self):
        check = self.assertRedirectsWithAllMethods
        check(reverse('delivery:order'))
        check(reverse('delivery:meal'))
        meal = Component.objects.first()
        meal_id = meal.id
        check(reverse('delivery:meal_id', kwargs={'id': meal_id}))
        check(reverse('delivery:routes'))
        check(reverse('delivery:kitchen_count'))
        check(reverse('delivery:kitchen_count_date', kwargs={
            'year': 2016,
            'month': 11,
            'day': 30
        }))
        check(reverse('delivery:mealLabels'))
        check(reverse('delivery:route_sheet', kwargs={'pk': 1}))
        check(reverse('delivery:refresh_orders'))


class OrderlistViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ['sample_data']

    def setUp(self):
        super(OrderlistViewTestCase, self).setUp()
        self.force_login()

    def test_can_filter_orders_by_client_names(self):
        # Setup
        Order.objects.all().update(delivery_date=tz.now())
        url = reverse('delivery:order')
        # Run
        response = self.client.get(url, {'client_name': 'john'})
        # Check
        self.assertEqual(
            set(response.context['orders']),
            set(Order.objects.filter(
                Q(client__member__firstname__icontains='john') |
                Q(client__member__lastname__icontains='john')
            )))

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:order')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:order')
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class KitchenCountOrderFilterTestCase(SousChefTestMixin, TestCase):
    fixtures = ['sample_data']

    def setUp(self):
        super(KitchenCountOrderFilterTestCase, self).setUp()
        self.factory = RequestFactory()

    def test_can_filter_orders_by_client_names(self):
        # Setup
        queryset = Order.objects.all()
        request_1 = self.factory.get('/')
        request_2 = self.factory.get('/', {'client_name': 'john doe'})
        # Run
        filterset_1 = KitchenCountOrderFilter(request_1.GET, queryset)
        filterset_2 = KitchenCountOrderFilter(request_2.GET, queryset)
        # Check
        self.assertEqual(filterset_1.qs.count(), queryset.count())
        self.assertTrue(filterset_2.qs.count())
        self.assertEqual(
            set(filterset_2.qs),
            set(Order.objects.filter(
                Q(client__member__firstname__icontains='john') |
                Q(client__member__firstname__icontains='doe') |
                Q(client__member__lastname__icontains='john') |
                Q(client__member__lastname__icontains='doe')
            )))


class RoutesInformationViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ['sample_data']

    def setUp(self):
        super(RoutesInformationViewTestCase, self).setUp()
        self.force_login()

    def test_can_embed_additional_route_sheet_information_when_printing(self):
        # Setup
        url = reverse('delivery:routes')
        # Run
        response_1 = self.client.get(url)
        response_2 = self.client.get(url, {'print': 'yes'})
        # Check
        self.assertNotIn('routes_dict', response_1)
        self.assertIn('routes_dict', response_2)

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:routes')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:routes')
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class KitchenCountViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ['sample_data']

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:kitchen_count')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:kitchen_count')

        # main dish and its ingredients today
        main_dishes = Component.objects.filter(name='Ginger pork')
        main_dish = main_dishes[0]
        dish_ingredients = Component.get_recipe_ingredients(
            main_dish.id)
        for ing in dish_ingredients:
            ci = Component_ingredient(
                component=main_dish,
                ingredient=ing,
                date=datetime.date.today())
            ci.save()
        # Add sides ingredient
        sides_component = Component.objects.get(
            component_group=COMPONENT_GROUP_CHOICES_SIDES)
        sides_ingredient = Ingredient.objects.get(name='Brussel sprouts')
        ci = Component_ingredient(
            component=sides_component,
            ingredient=sides_ingredient,
            date=datetime.date.today())
        ci.save()

        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class MealLabelsViewTestCase(SousChefTestMixin, TestCase):
    fixtures = ['sample_data']

    def test_redirects_users_who_do_not_have_read_permission(self):
        # Setup
        User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:mealLabels')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_read_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:mealLabels')
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)


class RefreshOrderViewTestCase(SousChefTestMixin, TestCase):

    def test_redirects_users_who_do_not_have_edit_permission(self):
        # Setup
        user = User.objects.create_user(
            username='foo', email='foo@example.com', password='secure')
        user.is_staff = True
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:refresh_orders')
        # Run & check
        self.assertRedirectsWithAllMethods(url)

    def test_allow_access_to_users_with_edit_permission(self):
        # Setup
        user = User.objects.create_superuser(
            username='foo', email='foo@example.com', password='secure')
        user.save()
        self.client.login(username='foo', password='secure')
        url = reverse('delivery:refresh_orders')
        # Run
        response = self.client.get(url)
        # Check
        self.assertEqual(response.status_code, 200)

    def test_refresh_create_only_if_scheduled_today(self):
        """
        Refs bug #734.
        If a client is ongoing but don't have scheduled delivery,
        then don't create his order.
        """
        meals_default = {}
        data = {
            'main_dish_{}_quantity': 2,
            'size_{}': 'R',
            'dessert_{}_quantity': 0,
            'diabetic_dessert_{}_quantity': 0,
            'fruit_salad_{}_quantity': 1,
            'green_salad_{}_quantity': 1,
            'pudding_{}_quantity': 1,
            'compote_{}_quantity': 0,
        }
        for day, _ in DAYS_OF_WEEK:
            for key_tmpl, value in data.items():
                meals_default[key_tmpl.format(day)] = value

        ongoing_clients = ClientFactory.create_batch(
            10, status=Client.ACTIVE, delivery_type='O',
            meal_default_week=meals_default,
            route=RouteFactory()
        )

        # The meal schedule should exclude today.
        schedule_delivery = []
        weekday = datetime.date.today().weekday()
        for i, t in enumerate(DAYS_OF_WEEK):
            if i != weekday:
                schedule_delivery.append(t[0])

        for c in ongoing_clients:
            c.set_simple_meals_schedule(schedule_delivery)

        self.force_login()
        url = reverse('delivery:refresh_orders')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            Order.objects.all().count(),
            0  # No order should be created.
        )


class ExcludeMalconfiguredClientsTestCase(SousChefTestMixin, TestCase):
    """
    Test 4 clients:
    - c_valid: correctly configured
               id==10, avoid veggie_10->chicken_10, avoid wine_10
    - c_nr:    malconfigured: route==None
               id==20, avoid veggie_20->chicken_20, avoid wine_20
    - c_ng:    malconfigured: not .isgeolocalized
               id==30, avoid veggie_30->chicken_30, avoid wine_30
    - c_nrng:  malconfigured: route==None and not .isgeolocalized
               id==40, avoid veggie_40->chicken_40, avoid wine_40

    Test meal = chicken_10 + wine_10 + chicken_20 + wine_20 + ...

    c_nr, c_ng and c_nrng should be excluded.
    """

    @classmethod
    def setUpTestData(cls):
        meal_default_week = {
            "main_dish_friday_quantity": 1,
            "main_dish_monday_quantity": 1,
            "main_dish_saturday_quantity": 1,
            "main_dish_sunday_quantity": 1,
            "main_dish_thursday_quantity": 1,
            "main_dish_tuesday_quantity": 1,
            "main_dish_wednesday_quantity": 1,

            "size_friday": "L",
            "size_monday": "L",
            "size_saturday": "L",
            "size_sunday": "L",
            "size_thursday": "L",
            "size_tuesday": "L",
            "size_wednesday": "L",

            "dessert_friday_quantity": 1,
            "dessert_monday_quantity": 1,
            "dessert_saturday_quantity": 1,
            "dessert_sunday_quantity": 1,
            "dessert_thursday_quantity": 1,
            "dessert_tuesday_quantity": 1,
            "dessert_wednesday_quantity": 1,
        }
        cls.route1 = RouteFactory()
        cls.route2 = RouteFactory()
        cls.c_valid = ClientFactory(
            pk=10,
            status=Client.ACTIVE,
            delivery_type="O",  # ongoing
            route=cls.route1,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Valid",
                lastname="Valid",
                address=AddressFactory()
            )
        )
        cls.c_nr = ClientFactory(
            pk=20,
            status=Client.ACTIVE,
            delivery_type="O",  # ongoing
            route=None,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Nronly",
                lastname="Nronly",
                address=AddressFactory()
            )
        )
        cls.c_ng = ClientFactory(
            pk=30,
            status=Client.ACTIVE,
            delivery_type="O",  # ongoing
            route=cls.route2,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Ngonly",
                lastname="Ngonly",
                address=AddressFactory(
                    latitude=None
                )
            )
        )
        cls.c_nrng = ClientFactory(
            pk=40,
            status=Client.ACTIVE,
            delivery_type="O",  # ongoing
            route=None,
            meal_default_week=meal_default_week,
            member=MemberFactory(
                firstname="Nrng",
                lastname="Nrng",
                address=AddressFactory(
                    longitude=None
                )
            )
        )

        cls.main_dish = ComponentFactory(
            name="coq au vin",
            component_group="main_dish")
        cls.sides = ComponentFactory(
            name="sides",
            component_group="sides")

        cls.ingred_chickens = []
        cls.ingred_wines = []
        for client in (cls.c_valid, cls.c_nr, cls.c_ng, cls.c_nrng):
            # Ensure that orders are created when refrshing orders.
            client.set_simple_meals_schedule([
                'monday', 'tuesday', 'wednesday', 'thursday', 'friday',
                'saturday', 'sunday'])

            # client-specific ingredient (for testing clashes)
            this_chicken = IngredientFactory(
                name="chicken_{0}".format(client.id),
                ingredient_group="meat")
            this_wine = IngredientFactory(
                name="wine_{0}".format(client.id),
                ingredient_group="oils_and_sauces")

            # add them to main_dish
            ComponentIngredientFactory(
                component=cls.main_dish,
                ingredient=this_chicken)
            ComponentIngredientFactory(
                component=cls.main_dish,
                ingredient=this_wine)

            # all special veggies, special winehaters
            this_incompatib = IncompatibilityFactory(
                restricted_item=RestrictedItemFactory(
                    name="veggie_{0}".format(client.id),
                    restricted_item_group="meat"
                ),
                ingredient=this_chicken
            )
            Restriction.objects.create(
                client=client,
                restricted_item=this_incompatib.restricted_item)
            Client_avoid_ingredient.objects.create(
                client=client,
                ingredient=this_wine)

            # keep track of these chickens and wines
            cls.ingred_chickens.append(this_chicken)
            cls.ingred_wines.append(this_wine)

    def setUp(self):
        self.force_login()

    def _refresh_orders(self):
        return self.client.get(reverse('delivery:refresh_orders'))

    def _today_meal(self):
        return self.client.post(reverse('delivery:meal'), {
            'maindish': [str(self.main_dish.id)],
            '_update': ['Next: Print Kitchen Count'],
            'ingredients': map(
                lambda ingred: str(ingred.id),
                self.ingred_chickens + self.ingred_wines
            ),
            'sides_ingredients': map(
                lambda ingred: str(ingred.id),
                self.ingred_chickens
            )
        })

    def test_step_1__review_orders(self):
        response = self.client.get(reverse('delivery:order'))
        self.assertEqual(
            len(response.context['orders']), 0
        )
        self.assertEqual(
            response.content.count(
                b'<i class="warning sign red icon"></i>'
            ), 0
        )
        # refresh to create orders
        response = self._refresh_orders()
        self.assertEqual(
            len(response.context['orders']), 4
        )
        self.assertEqual(
            response.content.count(
                b'<i class="warning sign red icon"></i>'
            ), 3
        )

    def test_step_3__component_table(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(response, reverse("delivery:meal"))
        response = self.client.get(reverse("delivery:kitchen_count"))
        component_lines = response.context['component_lines']
        main_dish_component_line = next(
            cl for cl in component_lines
            if cl.component_group == ugettext_lazy('Main Dish')
        )
        self.assertEqual(main_dish_component_line.rqty, 0)
        # only c_valid
        self.assertEqual(main_dish_component_line.lqty, 1)

    def test_step_3__clashing_ingredients_restrictions_table(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(response, reverse("delivery:meal"))
        response = self.client.get(reverse("delivery:kitchen_count"))
        meal_lines = response.context['meal_lines']

        # contains c_valid (10)
        ml_valid_clash = next(
            ml for ml in meal_lines
            if 'chicken_10' in ml.ingr_clash and
            'wine_10' in ml.ingr_clash
        )
        self.assertEqual(ml_valid_clash.rqty, '0')
        self.assertEqual(ml_valid_clash.lqty, '1')
        ml_valid_restrict = next(
            ml for ml in meal_lines
            if 'veggie_10' in ml.rest_item
        )
        self.assertIn('Valid', ml_valid_restrict.client)

        # doesn't contain c_nronly (20)
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'chicken_20' in ml.ingr_clash and
            'wine_20' in ml.ingr_clash
        ))
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'veggie_20' in ml.rest_item
        ))

        # doesn't contain c_ngonly (30)
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'chicken_30' in ml.ingr_clash and
            'wine_30' in ml.ingr_clash
        ))
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'veggie_30' in ml.rest_item
        ))

        # doesn't contain c_nrng (40)
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'chicken_40' in ml.ingr_clash and
            'wine_40' in ml.ingr_clash
        ))
        self.assertRaises(StopIteration, lambda: next(
            ml for ml in meal_lines
            if 'veggie_40' in ml.rest_item
        ))

        # TOTAL SPECIALS
        ml_last = meal_lines[-1]
        self.assertEqual(ml_last.ingr_clash, "TOTAL SPECIALS")
        self.assertEqual(ml_last.rqty, '0')
        self.assertEqual(ml_last.lqty, '1')

    def test_step_3__labels(self):
        _ = self._refresh_orders()
        response = self._today_meal()
        self.assertRedirects(response, reverse("delivery:meal"))
        response = self.client.get(reverse("delivery:kitchen_count"))
        num_labels = response.context['num_labels']
        self.assertEqual(num_labels, 1)  # only c_valid

    def test_step_4__routes_before_organizing(self):
        """
        Should ignore route==None and non geolocalized clients.
        When the delivery history doesn't exist or it's invalid,
        don't let route sheet and route sheets work.
        """
        _ = self._refresh_orders()
        _ = self._today_meal()
        response = self.client.get(reverse("delivery:routes"))
        route_orders_tuples = response.context['route_details']
        route_orders_dict = dict(map(
            lambda t: (t[0], t[1:]), route_orders_tuples
        ))
        self.assertIn(self.route1, route_orders_dict)
        self.assertEqual(route_orders_dict[self.route1][0], 1)
        self.assertIn(self.route2, route_orders_dict)
        self.assertEqual(route_orders_dict[self.route2][0], 0)
        # Assert not configured
        self.assertFalse(response.context['all_configured'])
        self.assertFalse(DeliveryHistory.objects.filter(
            route=self.route1,
            date=tz.datetime.today()
        ).exists())
        self.assertFalse(DeliveryHistory.objects.filter(
            route=self.route2,
            date=tz.datetime.today()
        ).exists())
        # Assert print doesn't work
        response = self.client.get(reverse("delivery:routes"),
                                   {'print': 'yes'})
        self.assertEqual(response.status_code, 404)
        # Assert route sheets don't exist
        response = self.client.get(
            reverse("delivery:route_sheet", args=[self.route1.pk])
        )
        self.assertEqual(response.status_code, 404)
        response = self.client.get(
            reverse("delivery:route_sheet", args=[self.route2.pk])
        )
        self.assertEqual(response.status_code, 404)

        # Try invalid DeliveryHistory
        DeliveryHistory.objects.create(
            route=self.route1,
            date=tz.datetime.today(),
            # As long as this sequence is not exactly same as the clients',
            # it's always invalid, even if it contains all the clients.
            client_id_sequence=[999999, 888888, self.c_valid.pk]
        )
        # Assert print doesn't work
        response = self.client.get(reverse("delivery:routes"),
                                   {'print': 'yes'})
        self.assertEqual(response.status_code, 404)

    def test_step_4__routes_after_organizing(self):
        """
        Should ignore route==None and not geolocalized
        """
        _ = self._refresh_orders()
        _ = self._today_meal()
        # Work on Route 1
        response = self.client.post(
            reverse("delivery:create_delivery_of_today", args=[self.route1.pk])
        )
        self.assertRedirects(
            response,
            reverse("delivery:edit_delivery_of_today", args=[self.route1.pk])
        )
        self.assertTrue(DeliveryHistory.objects.filter(
            route=self.route1,
            date=tz.datetime.today()
        ).exists())
        response = self.client.get(
            reverse("delivery:edit_delivery_of_today", args=[self.route1.pk])
        )
        self.assertEqual(response.status_code, 200)
        response = self.client.post(
            reverse("delivery:edit_delivery_of_today", args=[self.route1.pk]),
            {
                'vehicle': 'walking',
                'comments': 'My bicycle is broken',
                'client_id_sequence': json.dumps([self.c_valid.pk])
            }
        )
        self.assertRedirects(
            response,
            reverse("delivery:routes")
        )

        # Assert configured because Route 2 is empty.
        response = self.client.get(reverse("delivery:routes"))
        self.assertTrue(response.context['all_configured'])

        # Assert Route 2 doesn't allow the creation of delivery
        response = self.client.post(
            reverse("delivery:create_delivery_of_today", args=[self.route2.pk])
        )
        self.assertEqual(response.status_code, 404)
        self.assertFalse(DeliveryHistory.objects.filter(
            route=self.route2,
            date=tz.datetime.today()
        ).exists())
        response = self.client.get(
            reverse("delivery:edit_delivery_of_today", args=[self.route2.pk])
        )
        self.assertEqual(response.status_code, 404)
        response = self.client.post(
            reverse("delivery:edit_delivery_of_today", args=[self.route2.pk])
        )
        self.assertEqual(response.status_code, 404)

        # Assert print works
        response = self.client.get(reverse("delivery:routes"),
                                   {'print': 'yes'})
        self.assertEqual(response.status_code, 200)
        # Check print result
        routes_dict = json.loads(response['routes_dict'])
        # only route1 has one client
        route1 = routes_dict[str(self.route1.id)]
        self.assertEqual(len(route1['detail_lines']), 1)
        # route2 doesn't exist.
        self.assertNotIn(str(self.route2.id), routes_dict)

        # check output page : use data in header because cannot read PDF
        route_name_list = \
            [item['route'] for item in routes_dict.values()]
        self.assertIn(self.route1.name, route_name_list)
        self.assertNotIn(self.route2.name, route_name_list)

        # Assert route sheets exist and check route sheets
        # Route 1
        response1 = self.client.get(
            reverse("delivery:route_sheet", args=[self.route1.pk])
        )
        self.assertEqual(response1.status_code, 200)
        # 1 dessert 1 L main_dish
        summary_line1 = response1.context['summary_lines']
        main_dish_line1 = next(
            l for l in summary_line1 if l.component_group == 'main_dish'
        )
        self.assertEqual(main_dish_line1.rqty, 0)
        self.assertEqual(main_dish_line1.lqty, 1)
        dessert_line1 = next(
            l for l in summary_line1 if l.component_group == 'dessert'
        )
        self.assertEqual(dessert_line1.rqty, 1)
        # 1 client
        detail_line1 = response1.context['detail_lines']
        self.assertEqual(len(detail_line1), 1)
        self.assertEqual(detail_line1[0].firstname, 'Valid')
        self.assertEqual(detail_line1[0].lastname, 'Valid')

        # Route 2 - not found
        response2 = self.client.get(
            reverse("delivery:route_sheet", args=[self.route2.pk])
        )
        self.assertEqual(response2.status_code, 404)


class RouteSheetReportTestCase(SousChefTestMixin, TestCase):
    # PDF route sheets report generation
    fixtures = ['sample_data']

    def setUp(self):
        self.force_login()

    def test_route_sheet_report(self):
        # generate orders today
        Order.objects.auto_create_orders(
            datetime.date.today(), Client.active.all())

        # list all the routes
        response = self.client.get(reverse("delivery:routes"))
        self.assertEqual(response.status_code, 200)
        # get details of each route : {route:(order_count, ...), ...}
        route_orders_dict = dict(map(
            lambda t: (t[0], t[1:]),
            response.context['route_details']
        ))

        # create history and organize clients for the non empty routes
        for route, num in route_orders_dict.items():
            if num[0] > 0:

                # route has clients with orders : create delivery history
                response = self.client.post(
                    reverse("delivery:create_delivery_of_today",
                            args=[route.pk])
                )
                self.assertRedirects(
                    response,
                    reverse("delivery:edit_delivery_of_today",
                            args=[route.pk])
                )

                # get the clients in the route
                response = self.client.get(
                    reverse("delivery:edit_delivery_of_today",
                            args=[route.pk])
                )
                self.assertEqual(response.status_code, 200)
                client_pks = [
                    item.pk for item
                    in response.context['clients_on_delivery_history']]

                # organize the clients in the route
                response = self.client.post(
                    reverse("delivery:edit_delivery_of_today",
                            args=[route.pk]),
                    {
                        'vehicle': 'walking',
                        'comments': 'Nice day !',
                        'client_id_sequence': json.dumps(client_pks)
                    }
                )
                self.assertRedirects(
                    response, reverse("delivery:routes"))

        # generate the PDF route sheets of all the routes
        response = self.client.get(reverse("delivery:routes"),
                                   {'print': 'yes'})
        self.assertEqual(response.status_code, 200)
        self.assertTrue('ReportLab' in repr(response.content))



# -*- coding: utf-8 -*-

import rules
from rules.predicates import is_authenticated
from rules.predicates import is_staff
from rules.predicates import is_superuser


# Note: only two global "permissions" are considered in this project for now.
# "read" and "edit". Should fine-grained permissions be needed, these basic
# permissions should be removed and replaced by permissions associated with
# each app.
rules.add_perm('sous_chef.edit', is_authenticated & is_superuser)
rules.add_perm('sous_chef.read', is_authenticated & is_staff)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-05 19:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Component',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('component_group', models.CharField(choices=[('main dish', 'main dish'), ('vegetable', 'Vegetable'), ('side dish', 'side dish'), ('seasonal', 'Seasonal')], max_length=100, verbose_name='component group')),
            ],
            options={
                'verbose_name_plural': 'components',
            },
        ),
        migrations.CreateModel(
            name='Component_ingredient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('component', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Component', verbose_name='component')),
            ],
        ),
        migrations.CreateModel(
            name='Incompatibility',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='Ingredient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
            ],
            options={
                'verbose_name_plural': 'meals',
            },
        ),
        migrations.CreateModel(
            name='Menu',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='date')),
            ],
            options={
                'verbose_name_plural': 'menus',
            },
        ),
        migrations.CreateModel(
            name='Menu_component',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('component', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Component', verbose_name='component')),
                ('menu', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Menu', verbose_name='menu')),
            ],
        ),
        migrations.CreateModel(
            name='Restricted_item',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('restricted_item_group', models.CharField(choices=[('meat', 'Meat'), ('vegetables', 'Vegetables'), ('seafood', 'Seafood'), ('seeds and nuts', 'Seeds and nuts'), ('other', 'Other')], max_length=100, verbose_name='restricted item group')),
            ],
            options={
                'verbose_name_plural': 'restricted items',
            },
        ),
        migrations.AddField(
            model_name='incompatibility',
            name='ingredient',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Ingredient', verbose_name='ingredient'),
        ),
        migrations.AddField(
            model_name='incompatibility',
            name='restricted_item',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Restricted_item', verbose_name='restricted item'),
        ),
        migrations.AddField(
            model_name='component_ingredient',
            name='ingredient',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='meal.Ingredient', verbose_name='ingredient'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-28 18:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0030_route_deliveryhistory'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client_option',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Client', verbose_name='client'),
        ),
        migrations.AlterField(
            model_name='client_option',
            name='option',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Option', verbose_name='option'),
        ),
    ]



from django.core.management.base import BaseCommand
from souschef.member.factories import NoteFactory


class Command(BaseCommand):
    help = 'Creates notes related to members.'

    def handle(self, *args, **options):
        notes = NoteFactory.create_batch(20)
        self.stdout.write(
            self.style.SUCCESS(
                'Successfully created notes "%s"' % notes
            )
        )



import csv

from django.core.management.base import BaseCommand
from souschef.member.models import Member, Address, Contact, EMAIL, HOME


class Command(BaseCommand):
    help = 'Data: import clients from given csv file.'

    ROW_MID = 0
    ROW_ADDRESS1 = 3
    ROW_APARTMENT = 4
    ROW_CITY = 5
    ROW_POSTAL_CODE = 6
    ROW_PHONE = 7
    ROW_EMAIL = 8

    def add_arguments(self, parser):
        parser.add_argument(
            '--file',
            default=False,
            help='Import mock data instead of actual data',
        )

    def handle(self, *args, **options):
        if options['file']:
            file = 'mock_addresses.csv'
        else:
            file = 'clients_address.csv'

        with open(file) as f:
            reader = csv.reader(f, delimiter=';')
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    # if member.address is not None:
                    #    member.address.delete()
                    address = Address.objects.create(
                        street=row[self.ROW_ADDRESS1],
                        apartment=row[self.ROW_APARTMENT],
                        city=row[self.ROW_CITY],
                        postal_code=row[self.ROW_POSTAL_CODE],
                    )

                    member.address = address
                    member.save()

                    contacts = Contact.objects.filter(member=member)
                    contacts.delete()

                    if row[self.ROW_PHONE] != '':
                        Contact.objects.create(
                            type=HOME,
                            value=row[self.ROW_PHONE],
                            member=member
                        )
                    if row[self.ROW_EMAIL] != '':
                        Contact.objects.create(
                            type=EMAIL,
                            value=row[self.ROW_EMAIL],
                            member=member
                        )

                except Member.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing member'))



# usage : python dataexec.py [santropolFeast.settingsSPECIAL]
import os
import sys


def run():
    if len(sys.argv) > 1:
        settings = sys.argv[1]
    else:
        settings = 'souschef.sous_chef.settings'
    os.environ['DJANGO_SETTINGS_MODULE'] = settings
    import django
    django.setup()
    from dataload import insert_all
    insert_all()

run()



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-11-25 20:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0020_auto_20161122_0458'),
    ]

    operations = [
        migrations.AlterField(
            model_name='address',
            name='postal_code',
            field=models.CharField(max_length=7, verbose_name='postal code'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-08-03 11:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0004_fix004d'),
    ]

    operations = [
        migrations.AlterField(
            model_name='component',
            name='component_group',
            field=models.CharField(choices=[('main_dish', 'Main Dish'), ('dessert', 'Dessert'), ('diabetic', 'Diabetic Dessert'), ('fruit_salad', 'Fruit Salad'), ('green_salad', 'Green Salad'), ('pudding', 'Pudding'), ('compote', 'Compote')], max_length=100, verbose_name='component group'),
        ),
        migrations.AlterField(
            model_name='component',
            name='name',
            field=models.CharField(max_length=50, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='ingredient',
            name='ingredient_group',
            field=models.CharField(choices=[('meat', 'Meat'), ('dairy', 'Dairy'), ('fish', 'Fish'), ('seafood', 'Seafood'), ('veggies_and_fruits', 'Veggies and fruits'), ('legumineuse', 'Legumineuse'), ('grains', 'Grains'), ('fresh_herbs', 'Fresh herbs'), ('spices', 'Spices'), ('dry_and_canned_goods', 'Dry and canned goods'), ('oils_and_sauces', 'Oils and sauces')], max_length=100, verbose_name='ingredient group'),
        ),
        migrations.AlterField(
            model_name='ingredient',
            name='name',
            field=models.CharField(max_length=50, unique=True, verbose_name='name'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-03-05 15:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0036_member_addresses'),
    ]

    operations = [
        migrations.AlterField(
            model_name='member',
            name='address',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='member.Address', verbose_name='address'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-12 19:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0009_auto_20161012_1919'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='creation_date',
            field=models.DateField(auto_now_add=True, verbose_name='creation date'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-26 17:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0006_client_meal_default_week'),
    ]

    operations = [
        migrations.AddField(
            model_name='address',
            name='distance',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='lat',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='lon',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-01 20:07
from __future__ import unicode_literals

import annoying.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0032_relationship'),
    ]

    operations = [
        migrations.AlterField(
            model_name='relationship',
            name='extra_fields',
            field=annoying.fields.JSONField(blank=True, default={}),
        ),
        migrations.AlterField(
            model_name='relationship',
            name='remark',
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name='relationship',
            name='type',
            field=annoying.fields.JSONField(blank=True, default=[]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-13 18:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0026_change_to_emergency_contacts'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='billing_member',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='member.Member', verbose_name='billing member'),
        ),
        migrations.AlterField(
            model_name='member',
            name='address',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='member.Address', verbose_name='address'),
        ),
        migrations.AlterField(
            model_name='route',
            name='vehicle',
            field=models.CharField(choices=[('cycling', 'Cycling'), ('walking', 'Walking'), ('driving', 'Driving')], default='cycling', max_length=20, verbose_name='vehicle'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-20 20:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0016_auto_20160912_1509'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='route',
            options={'ordering': ['name'], 'verbose_name_plural': 'Routes'},
        ),
        migrations.AlterField(
            model_name='client_option',
            name='value',
            field=models.CharField(max_length=255, null=True, verbose_name='value'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-20 23:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0012_clientscheduledstatus'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clientscheduledstatus',
            name='change_date',
            field=models.DateField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='clientscheduledstatus',
            name='change_state',
            field=models.CharField(choices=[('START', 'Start'), ('END', 'End')], default='START', max_length=5),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-03 20:52
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0009_auto_20160728_1443'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='note',
            name='author',
        ),
        migrations.RemoveField(
            model_name='note',
            name='member',
        ),
        migrations.DeleteModel(
            name='Note',
        ),
    ]



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.note.views import (
    NoteAdd,
    NoteBatchToggle,
    NoteDeleteView,
    NoteEditView,
    NoteList,
)

from souschef.note.views import mark_as_read, mark_as_unread

app_name = "note"

urlpatterns = [
    url(_(r'^$'), NoteList.as_view(), name='note_list'),
    url(_(r'^read/(?P<id>\d+)/$'),
        mark_as_read, name='read'),
    url(_(r'^unread/(?P<id>\d+)/$'),
        mark_as_unread, name='unread'),
    url(_(r'^add/$'), NoteAdd.as_view(), name='note_add'),
    url(_(r'^batch_toggle/$'),
        NoteBatchToggle.as_view(), name='batch_toggle'),
    url(_(r'^edit/(?P<pk>\d+)/$'), NoteEditView.as_view(), name='edit'),
    url(_(r'^delete/(?P<pk>\d+)/$'), NoteDeleteView.as_view(), name='note_delete'),
]



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.db.models.functions import Lower

from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES_MAIN_DISH, Component, Ingredient)


class DishIngredientsForm(forms.Form):
    maindish = forms.ModelChoiceField(
        label=_("Today's main dish:"),
        queryset=Component.objects.order_by(Lower('name')).filter(
            component_group=COMPONENT_GROUP_CHOICES_MAIN_DISH),
        widget=forms.Select(attrs={'class': 'ui dropdown maindish'}),
    )

    ingredients = forms.ModelMultipleChoiceField(
        label=_('Select main dish ingredients:'),
        queryset=Ingredient.objects.order_by(Lower('name')).all(),
        widget=forms.SelectMultiple(
            attrs={'class': 'ui fluid search dropdown mainingredients'}),
        required=False,
    )

    sides_ingredients = forms.ModelMultipleChoiceField(
        label=_('Select sides ingredients:'),
        queryset=Ingredient.objects.order_by(Lower('name')).all(),
        widget=forms.SelectMultiple(
            attrs={'class': 'ui fluid search dropdown sidesingredients'}),
        required=False,
    )

    def clean_sides_ingredients(self):
        data = self.cleaned_data['sides_ingredients']
        if not data:
            raise forms.ValidationError(
                _("Please choose some Sides ingredients"))
        return data



# coding=utf-8
import factory
import random

from souschef.member.models import (
    Member, Client, Contact, Route, Address,
    Relationship, GENDER_CHOICES, PAYMENT_TYPE,
    DELIVERY_TYPE, DAYS_OF_WEEK, RATE_TYPE,
    ClientScheduledStatus, DeliveryHistory
)
from souschef.meal.models import COMPONENT_GROUP_CHOICES


class AddressFactory (factory.DjangoModelFactory):

    class Meta:
        model = Address

    street = factory.Faker('street_address')
    apartment = factory.Faker('random_number')
    city = 'Montreal'
    postal_code = factory.Faker('postalcode', locale="en_CA")
    latitude = factory.LazyAttribute(
        lambda x: random.choice(["75.0", "75.1", "75.2"])
    )
    longitude = factory.LazyAttribute(
        lambda x: random.choice(["40.2", "40.1", "40.0"])
    )


class MemberFactory (factory.DjangoModelFactory):

    class Meta:
        model = Member

    firstname = factory.Faker('first_name')
    lastname = factory.Faker('last_name')

    address = factory.SubFactory(AddressFactory)
    work_information = factory.Faker('company')
    contact = factory.RelatedFactory(
        'member.factories.ContactFactory',
        'member'
    )


class RouteFactory(factory.DjangoModelFactory):

    class Meta:
        model = Route

    name = factory.Faker('name')


class DeliveryHistoryFactory(factory.DjangoModelFactory):
    class Meta:
        model = DeliveryHistory


def generate_json():
    json = {}
    for day, translation in DAYS_OF_WEEK:
        json['size_{}'.format(day)] = random.choice(['L', 'R'])
        for meal, Meal in COMPONENT_GROUP_CHOICES:
            json['{}_{}_quantity'.format(meal, day)] = random.choice([0, 1])
    return json


class ClientFactory (factory.DjangoModelFactory):

    class Meta:
        model = Client

    member = factory.SubFactory(MemberFactory)
    billing_member = member
    billing_payment_type = factory.LazyAttribute(
        lambda x: random.choice(PAYMENT_TYPE)[0]
    )
    rate_type = factory.LazyAttribute(
        lambda x: random.choice(RATE_TYPE)[0]
    )
    member = member
    status = factory.LazyAttribute(
        lambda x: random.choice(
            Client.CLIENT_STATUS)[0]
    )
    language = factory.LazyAttribute(
        lambda x: random.choice(
            Client.LANGUAGES)[0]
    )
    alert = factory.Faker('sentence')
    delivery_type = factory.LazyAttribute(
        lambda x: random.choice(DELIVERY_TYPE)[0]
    )
    gender = factory.LazyAttribute(
        lambda x: random.choice(GENDER_CHOICES)[0]
    )
    birthdate = factory.Faker('date')
    route = factory.LazyAttribute(
        lambda x: random.choice(Route.objects.all())
    )

    delivery_note = factory.Faker('sentence')

    meal_default_week = factory.LazyAttribute(lambda x: generate_json())


def random_combination(iterable, r):
    """
    Random selection from itertools.combinations(iterable, r)
    From Python docs (itertools)
    """
    pool = tuple(iterable)
    n = len(pool)
    indices = sorted(random.sample(range(n), r))
    return tuple(pool[i] for i in indices)


class RelationshipFactory(factory.DjangoModelFactory):
    class Meta:
        model = Relationship

    client = factory.SubFactory(ClientFactory)
    member = factory.SubFactory(MemberFactory)
    nature = factory.LazyAttribute(
        lambda x: random.choice(['friends', 'family', 'coworkers'])
    )
    type = factory.LazyAttribute(
        lambda x: list(random_combination(
            map(lambda tup: tup[0], Relationship.TYPE_CHOICES),
            random.randint(0, len(Relationship.TYPE_CHOICES))))
    )
    extra_fields = factory.LazyAttribute(
        lambda x: {})
    remark = factory.Faker('sentence')


class ContactFactory (factory.DjangoModelFactory):

    class Meta:
        model = Contact

    type = 'Home phone'
    value = factory.Sequence(lambda n: '514-555-%04d' % n)
    member = factory.SubFactory(MemberFactory)


class ClientScheduledStatusFactory(factory.DjangoModelFactory):

    class Meta:
        model = ClientScheduledStatus

    client = factory.SubFactory(ClientFactory)
    status_from = factory.LazyAttribute(
        lambda x: random.choice(
            Client.CLIENT_STATUS)[0]
    )
    status_to = factory.LazyAttribute(
        lambda x: random.choice(
            Client.CLIENT_STATUS)[0]
    )
    reason = factory.Faker('sentence')
    change_date = factory.Faker('date')
    change_state = factory.LazyAttribute(
        lambda x: random.choice(
            ClientScheduledStatus.CHANGE_STATUS)[0]
    )
    operation_status = ClientScheduledStatus.TOBEPROCESSED



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import string_concat

from souschef.member.views import (
    ClientWizard,
    ClientList,
    SearchMembers,
    ClientOrderList,
    ClientInfoView,
    ClientPaymentView,
    ClientAllergiesView,
    DeleteRestriction,
    DeleteClientOption,
    DeleteIngredientToAvoid,
    DeleteComponentToAvoid,
    geolocateAddress,
    ClientStatusScheduler,
    ClientStatusSchedulerReschedule,
    ClientStatusView,
    ClientStatusSchedulerDeleteView,
    ClientUpdateBasicInformation,
    ClientUpdateAddressInformation,
    ClientUpdatePaymentInformation,
    ClientUpdateDietaryRestriction,
    ClientUpdateRelationshipsInformation,
    RouteListView,
    RouteDetailView,
    RouteEditView,
    DeliveryHistoryDetailView,
    get_minimised_euclidean_distances_route_sequence,
)

from souschef.member.forms import (
    ClientBasicInformation, ClientAddressInformation,
    ClientPaymentInformation, ClientRestrictionsInformation
)
from souschef.member.formsets import CreateRelationshipFormset

from souschef.note.views import ClientNoteList, ClientNoteListAdd

app_name = "member"

member_forms = ({
    'name':        'basic_information',
    'step_url':    _('basic_information'),
    'create_form': ClientBasicInformation,
    'update_form': ClientUpdateBasicInformation
}, {
    'name':        'address_information',
    'step_url':    _('address_information'),
    'create_form': ClientAddressInformation,
    'update_form': ClientUpdateAddressInformation
}, {
    'name':        'relationships',
    'step_url':    _('relationships'),
    'create_form': CreateRelationshipFormset,
    'update_form': ClientUpdateRelationshipsInformation
}, {
    'name':        'payment_information',
    'step_url':    _('payment_information'),
    'create_form': ClientPaymentInformation,
    'update_form': ClientUpdatePaymentInformation,
}, {
    'name':        'dietary_restriction',
    'step_url':    _('dietary_restriction'),
    'create_form': ClientRestrictionsInformation,
    'update_form': ClientUpdateDietaryRestriction,
})

member_wizard = ClientWizard.as_view(
    list(map(lambda d: (d['name'], d['create_form']), member_forms)),
    i18n_url_names=list(map(
        lambda d: (d['name'], d['step_url']), member_forms
    )),
    url_name='member:member_step'
)


urlpatterns = [
    url(_(r'^create/$'), member_wizard, name='member_step'),
    url(_(r'^create/(?P<step>.+)/$'), member_wizard,
        name='member_step'),
    url(_(r'^list/$'), ClientList.as_view(), name='list'),
    url(_(r'^search/$'), SearchMembers.as_view(), name='search'),
    url(_(r'^view/(?P<pk>\d+)/orders$'),
        ClientOrderList.as_view(), name='list_orders'),
    url(_(r'^view/(?P<pk>\d+)/information$'),
        ClientInfoView.as_view(), name='client_information'),
    url(_(r'^view/(?P<pk>\d+)/billing$'),
        ClientPaymentView.as_view(), name='client_payment'),
    url(_(r'^view/(?P<pk>\d+)/preferences$'),
        ClientAllergiesView.as_view(), name='client_allergies'),
    url(_(r'^view/(?P<pk>\d+)/notes$'),
        ClientNoteList.as_view(), name='client_notes'),
    url(_(r'^view/(?P<pk>\d+)/notes/add$'),
        ClientNoteListAdd.as_view(), name='client_notes_add'),
    url(_(r'^geolocateAddress/$'), geolocateAddress, name='geolocateAddress'),
    url(_(r'^view/(?P<pk>\d+)/status$'),
        ClientStatusView.as_view(), name='client_status'),
    url(_(r'^client/(?P<pk>\d+)/status/scheduler$'),
        ClientStatusScheduler.as_view(), name='clientStatusScheduler'),
    url(_(r'^client/(?P<pk>\d+)/status/scheduler/reschedule/'
          r'(?P<scheduled_status_1_pk>\d+)'
          r',(?P<scheduled_status_2_pk>\d+)?'
          r'/$'),
        ClientStatusSchedulerReschedule.as_view(),
        name='clientStatusSchedulerReschedule'),
    url(
        r'^status/(?P<pk>\d+)/delete$',
        ClientStatusSchedulerDeleteView.as_view(),
        name='delete_status'
    ),
    url(_(r'^restriction/(?P<pk>\d+)/delete/$'),
        DeleteRestriction.as_view(), name='restriction_delete'),
    url(_(r'^client_option/(?P<pk>\d+)/delete/$'),
        DeleteClientOption.as_view(), name='client_option_delete'),
    url(_(r'^ingredient_to_avoid/(?P<pk>\d+)/delete/$'),
        DeleteIngredientToAvoid.as_view(), name='ingredient_to_avoid_delete'),
    url(_(r'^component_to_avoid/(?P<pk>\d+)/delete/$'),
        DeleteComponentToAvoid.as_view(), name='component_to_avoid_delete'),

    url(_(r'^routes/$'), RouteListView.as_view(), name='route_list'),
    url(_(r'^route/(?P<pk>\d+)/$'),
        RouteDetailView.as_view(), name='route_detail'),
    url(_(r'^route/(?P<pk>\d+)/edit/$'),
        RouteEditView.as_view(), name='route_edit'),
    url(_(r'^route/(?P<pk>\d+)/optimised_sequence/$'),
        get_minimised_euclidean_distances_route_sequence,
        name='route_get_optimised_sequence'),
    url(_(r'^route/(?P<route_pk>\d+)/(?P<date>\d{4}-\d{2}-\d{2})/$'),
        DeliveryHistoryDetailView.as_view(), name='delivery_history_detail'),
]

# Handle client update forms URL
for d in member_forms:
    urlpatterns.append(
        url(string_concat(_(r'^(?P<pk>\d+)/update/'), d['step_url'], '/$'),
            d['update_form'].as_view(),
            name='member_update_' + d['name'])
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-01-15 16:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0034_auto_20170816_0850'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='billing_payment_type',
            field=models.CharField(blank=True, choices=[(' ', '----'), ('3rd', '3rd Party'), ('credit', 'Credit card'), ('eft', 'EFT'), ('cash', 'Cash'), ('etransfert', 'e-Transfer'), ('cheque', 'Cheque')], max_length=10, null=True, verbose_name='Payment Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-27 22:51
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0007_auto_20160726_1703'),
    ]

    operations = [
        migrations.RenameField(
            model_name='address',
            old_name='lat',
            new_name='latitude',
        ),
        migrations.RenameField(
            model_name='address',
            old_name='lon',
            new_name='longitude',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-07-09 18:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0011_auto_20170419_1109'),
    ]

    operations = [
        migrations.AddField(
            model_name='note',
            name='is_deleted',
            field=models.BooleanField(default=False, verbose_name='Is deleted'),
        ),
    ]



from datetime import date
from django.core.management import call_command
from django.test import TestCase

from souschef.member.models import Member, Client, Address
from souschef.member.models import Option
from souschef.order.models import Order


class ImportMemberTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ['routes.json']

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command('importclients', file='mock_clients.csv')

    def test_import_member_info(self):
        self.assertEquals(10, Client.objects.all().count())
        self.assertEquals(10, Member.objects.all().count())
        dorothy = Client.objects.get(member__mid=94)
        created = dorothy.member.created_at
        self.assertEquals(dorothy.member.firstname, 'Dorothy')
        self.assertEquals(dorothy.member.lastname, 'Davis')
        self.assertEquals(dorothy.status, Client.ACTIVE)
        self.assertEquals(dorothy.birthdate, date(1938, 10, 10))
        self.assertEquals(
            date(created.year, created.month, created.day),
            date(2007, 9, 26))
        self.assertEquals(dorothy.gender, 'F')
        self.assertEquals(dorothy.language, 'EN')
        self.assertEquals(dorothy.delivery_type, 'O')
        self.assertEquals(dorothy.route.name, 'McGill')
        self.assertEquals(dorothy.delivery_note, 'Code entree: 17')
        self.assertEquals(dorothy.alert, 'communicate with her sister')

    def test_import_member_status(self):
        self.assertEquals(Client.active.all().count(), 2)
        self.assertEquals(Client.ongoing.all().count(), 2)
        self.assertEquals(Client.pending.all().count(), 1)
        self.assertEquals(Client.contact.all().count(), 6)

    def test_import_member_routes(self):
        self.assertEquals(
            Client.objects.filter(
                route__name='McGill').count(), 2)
        self.assertEquals(
            Client.objects.filter(
                route__name='Westmount').count(), 2)
        self.assertEquals(
            Client.objects.filter(
                route__name='Notre Dame de Grace').count(), 3)
        self.assertEquals(
            Client.objects.filter(
                route__name='Côte-des-Neiges').count(), 2)
        self.assertEquals(
            Client.objects.filter(
                route__name='Centre-Ville / Downtown').count(), 1)


class ImportMemberAddressesTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ['routes.json']

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command('importclients', file='mock_clients.csv')
        call_command('importaddresses', file='mock_addresses.csv')

    def test_import_member_addresses(self):
        self.assertEquals(Address.objects.all().count(), 10)
        dorothy = Member.objects.get(mid=94)
        self.assertEquals(dorothy.address.street, '2070 De Maisoneuve Ouest')
        self.assertEquals(dorothy.address.apartment, 'Apt.ABC')
        self.assertEquals(dorothy.address.postal_code, 'H3G1K9')
        self.assertEquals(dorothy.address.city, 'Montreal')
        self.assertEquals(dorothy.home_phone, '514-666-6666')
        self.assertEquals(dorothy.email, 'ddavis@example.org')

    def test_ut8_charset(self):
        norma = Member.objects.get(mid=91)
        self.assertEquals(norma.address.city, 'Montréal')
        robin = Member.objects.get(mid=99)
        self.assertEquals(robin.address.street, '29874 Côte-des-Neiges')


class ImportMemberRelationshipsTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ['routes.json']

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """
        call_command('importclients', file='mock_clients.csv')
        call_command('importrelationships', file='mock_relationships.csv')

    def test_import_relationship(self):
        self.assertEquals(10, Client.objects.all().count())
        self.assertEquals(12, Member.objects.all().count())
        dorothy = Client.objects.get(member__mid=94)
        self.assertEquals(str(dorothy.billing_member), 'Alice Cardona')
        self.assertEquals(dorothy.billing_member.rid, 2884)
        self.assertIn(
            dorothy.billing_member.pk,
            [c.pk for c in dorothy.emergency_contacts.all()]
        )
        self.assertIn(
            'daughter',
            [ce.relationship for ce in dorothy.emergencycontact_set.all()]
        )
        self.assertEquals(dorothy.client_referent.all().count(), 0)
        marie = Client.objects.get(member__mid=93)
        marion = Member.objects.get(rid=865)
        self.assertEquals(marie.client_referent.all().count(), 1)
        referencing = marie.client_referent.first()
        self.assertEquals(referencing.referent, marion)
        self.assertEquals(
            referencing.referent.work_information,
            'CLSC St-Louis du Parc'
        )
        self.assertEquals(referencing.referral_reason, 'Low mobility')


class ImportMemberMealsTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ['sample_data.json']

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """

        cls.food_preparation_puree = Option.objects.get(name='Puree all')
        cls.food_preparation_cut = Option.objects.get(name='Cut up meat')
        call_command('importclients', file='mock_clients.csv')
        call_command('importmeals', file='mock_meals.csv')

    def test_import_meals_schedule(self):
        dorothy = Client.objects.get(member__mid=94)
        # Dorothy is not registered to receive any meals
        self.assertEquals(
            dorothy.simple_meals_schedule,
            []
        )

        robert = Client.objects.get(member__mid=96)
        # Robert is registered to receive a meal 3 times a week
        self.assertEquals(
            robert.simple_meals_schedule,
            ["wednesday", "friday", "saturday"]
        )

        wednesday = robert.meals_schedule['wednesday']
        self.assertEquals(wednesday.get('fruit_salad'), 0)
        self.assertEquals(wednesday.get('green_salad'), 1)
        self.assertEquals(wednesday.get('main_dish'), 1)
        self.assertEquals(wednesday.get('size'), 'R')
        self.assertEquals(wednesday.get('dessert'), 1)
        self.assertEquals(wednesday.get('compote'), 0)
        self.assertEquals(wednesday.get('pudding'), 0)

    def test_import_meals_default(self):
        robert = Client.objects.get(member__mid=96)
        wednesday = robert.meals_schedule['wednesday']
        self.assertEquals(wednesday.get('fruit_salad'), 0)
        self.assertEquals(wednesday.get('green_salad'), 1)
        self.assertEquals(wednesday.get('main_dish'), 1)
        self.assertEquals(wednesday.get('size'), 'R')
        self.assertEquals(wednesday.get('dessert'), 1)
        self.assertEquals(wednesday.get('compote'), 0)
        self.assertEquals(wednesday.get('pudding'), 0)

    def test_import_food_preparation(self):
        robert = Client.objects.get(member__mid=96)
        self.assertEquals(robert.food_preparation.all().count(), 1)
        self.assertEquals(robert.food_preparation.first(),
                          self.food_preparation_cut)
        dorothy = Client.objects.get(member__mid=94)
        self.assertEquals(dorothy.food_preparation.all().count(), 2)

    def test_import_meal_labels(self):
        marie = Client.objects.get(member__mid=93)
        self.assertEquals(marie.notes.all().count(), 0)


class ImportMemberOrdersTestCase(TestCase):

    """
    Test data importation.
    """

    fixtures = ['sample_data.json']

    @classmethod
    def setUpTestData(cls):
        """
        Load the mock data files.
        It should import 10 clients.
        """

        call_command('importclients', file='mock_clients.csv')
        call_command('importorders', file='mock_orders.csv')

    def test_import_orders(self):
        dorothy = Client.objects.get(member__mid=94)
        orders = Order.objects.get_billable_orders_client(11, 2016, dorothy)
        # Dorothy must have one billable order
        self.assertEquals(
            orders.count(),
            1
        )
        self.assertEqual(orders.first().status, 'D')
        self.assertEqual(orders.first().delivery_date, date(2016, 11, 11))

    def test_import_order_items(self):
        dorothy = Client.objects.get(member__mid=94)
        order = Order.objects.get_billable_orders_client(
            11, 2016, dorothy
        ).first()
        items = order.orders
        main_dish = items.get(component_group='main_dish')
        self.assertEqual(main_dish.total_quantity, 2)
        self.assertEqual(main_dish.size, 'L')
        diabetic_dessert = items.get(component_group='diabetic')
        self.assertEqual(diabetic_dessert.total_quantity, 1)
        dessert = items.get(component_group='dessert')
        self.assertEqual(dessert.total_quantity, 1)



import csv

from django.core.management.base import BaseCommand
from souschef.member.models import Member, Address, Contact
from souschef.member.models import EMAIL, HOME, CELL, WORK


class Command(BaseCommand):
    help = 'Data: import clients from given csv file.'

    ROW_RID = 0
    ROW_ADDRESS1 = 3
    ROW_APARTMENT = 4
    ROW_CITY = 5
    ROW_POSTAL_CODE = 6
    ROW_HOME_PHONE = 7
    ROW_CELL_PHONE = 8
    ROW_WORK_PHONE = 9
    ROW_EXT = 10
    ROW_EMAIL = 11

    def add_arguments(self, parser):
        parser.add_argument(
            '--file',
            default=False,
            help='Import mock data instead of actual data',
        )

    def handle(self, *args, **options):
        if options['file']:
            file = 'mock_contactaddresses.csv'
        else:
            file = 'contact_address.csv'

        with open(file) as f:
            reader = csv.reader(f, delimiter=';')
            for row in reader:
                try:
                    member = Member.objects.get(rid=row[self.ROW_RID])

                    address = Address.objects.create(
                        street=row[self.ROW_ADDRESS1],
                        apartment=row[self.ROW_APARTMENT],
                        city=row[self.ROW_CITY],
                        postal_code=row[self.ROW_POSTAL_CODE],
                    )

                    member.address = address
                    member.save()

                    contacts = Contact.objects.filter(member=member)
                    contacts.delete()

                    if row[self.ROW_HOME_PHONE] != '' \
                            and len(row[self.ROW_HOME_PHONE]) >= 7:
                        Contact.objects.create(
                            type=HOME,
                            value=row[self.ROW_HOME_PHONE],
                            member=member
                        )
                    if row[self.ROW_CELL_PHONE] != ''\
                            and len(row[self.ROW_CELL_PHONE]) >= 7:
                        Contact.objects.create(
                            type=CELL,
                            value=row[self.ROW_CELL_PHONE],
                            member=member
                        )

                    if row[self.ROW_WORK_PHONE] != ''\
                            and len(row[self.ROW_WORK_PHONE]) >= 7:
                        work = row[self.ROW_WORK_PHONE]
                        if row[self.ROW_EXT] != '':
                            work += '  #' + row[self.ROW_EXT]

                        Contact.objects.create(
                            type=WORK,
                            value=work,
                            member=member
                        )
                    if row[self.ROW_EMAIL] != '':
                        Contact.objects.create(
                            type=EMAIL,
                            value=row[self.ROW_EMAIL],
                            member=member
                        )

                except Member.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing member'))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-10 21:36
from __future__ import unicode_literals

import annoying.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0029_member_address_fk_to_1to1'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeliveryHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='date of the delivery')),
                ('vehicle', models.CharField(choices=[('cycling', 'Cycling'), ('walking', 'Walking'), ('driving', 'Driving')], max_length=20, verbose_name='vehicle')),
                ('client_id_sequence', annoying.fields.JSONField(default=[], verbose_name='IDs of clients on this route (as a JSON list)')),
                ('comments', models.TextField(blank=True, null=True, verbose_name='comments')),
            ],
            options={
                'verbose_name': 'Delivery History',
                'ordering': ['route', '-date'],
                'verbose_name_plural': 'Delivery Histories',
            },
        ),
        migrations.AlterField(
            model_name='route',
            name='client_id_sequence',
            field=annoying.fields.JSONField(default=[], verbose_name='IDs of clients on this route (as a JSON list)'),
        ),
        migrations.AddField(
            model_name='deliveryhistory',
            name='route',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='delivery_histories', to='member.Route', verbose_name='route'),
        ),
        migrations.AlterUniqueTogether(
            name='deliveryhistory',
            unique_together=set([('route', 'date')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-01-10 19:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0024_auto_20161227_1819'),
    ]

    operations = [
        migrations.AddField(
            model_name='route',
            name='vehicle',
            field=models.CharField(choices=[('cycling', 'Cycling'), ('walking', 'Walking'), ('driving', 'Driving')], default='cycling', max_length=20, verbose_name='default vehicle'),
        ),
    ]
